# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class VirtualGateway(resource.Resource):

    resource_key = 'virtual_gateway'
    resources_key = 'virtual_gateways'
    base_path = '/dcaas/virtual-gateways'

    # capabilities
    allow_list = True
    allow_create = True
    allow_fetch = True
    allow_delete = True
    allow_commit = True

    _query_mapping = resource.QueryParameters(
        'project_id', 'name', 'vpc_id', 'local_ep_group_id', 'device_id',
        'redundant_device_id', 'type', 'ipsec_bandwidth', 'bgp_asn',
        'admin_state_up', project_id='tenant_id')

    # Properties
    #: ID of the direct connection.
    id = resource.Body('id')
    #: Indicates the virtual gateway name.
    name = resource.Body('name')
    #: Indicates the description of the virtual gateway.
    description = resource.Body('description')
    #: Specifies the ID of the VPC to be accessed.
    vpc_id = resource.Body('vpc_id')
    #: Specifies the ID of the local endpoint group that records CIDR blocks
    # of the VPC subnets.
    local_ep_group_id = resource.Body('local_ep_group_id')
    #: Specifies the ID of the physical device used by the virtual gateway.
    device_id = resource.Body('device_id')
    #: Specifies the ID of the redundant physical device used by
    # the virtual gateway.
    redundant_device_id = resource.Body('redundant_device_id')
    #: Specifies the virtual gateway type. The value can be default
    # or double ipsec.
    type = resource.Body('type')
    #: Specifies the bandwidth provided for IPsec VPN in Mbit/s.
    ipsec_bandwidth = resource.Body('ipsec_bandwidth', type=int)
    #: Specifies the BGP ASN of the virtual gateway.
    bgp_asn = resource.Body('bgp_asn', type=int)
    #: Specifies the administrative status of the virtual gateway.\
    # The value can be true or false.
    admin_state_up = resource.Body('admin_state_up', type=bool)
    #: Specifies the virtual gateway status. The value can be ACTIVE, DOWN,
    # BUILD, ERROR, PENDING_CREATE, PENDING_UPDATE, or PENDING_DELETE.
    status = resource.Body('status')
    #: Specifies the project ID.
    project_id = resource.Body('tenant_id')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from keystoneauth1 import adapter

import mock

from openstack.tests.unit import base

from otcextensions.sdk.auto_scaling.v1 import group

EXAMPLE = {
    "networks": [
        {
            "id": " a8327883-6b07-4497-9c61-68d03ee193a "
        }
    ],
    "detail": None,
    "scaling_group_name": "healthCheck",
    "scaling_group_id": "77a7a397-7d2f-4e79-9da9-6a35e2709150",
    "scaling_group_status": "INSERVICE",
    "scaling_configuration_id": "1d281494-6085-4579-b817-c1f813be835f",
    "scaling_configuration_name": "healthCheck",
    "current_instance_number": 0,
    "desire_instance_number": 1,
    "min_instance_number": 0,
    "max_instance_number": 500,
    "cool_down_time": 300,
    "lb_listener_id": "f06c0112570743b51c0e8fbe1f235bab",
    "security_groups": [
        {
            "id": "8a4b1d5b-0054-419f-84b1-5c8a59ebc829"
        }
    ],
    "create_time": "2015-07-23T02:46:29Z",
    "vpc_id": "863ccae2-ee85-4d27-bc5b-3ba2a198a9e2",
    "health_periodic_audit_method": "ELB_AUDIT",
    "health_periodic_audit_time": "5",
    "instance_terminate_policy": "OLD_CONFIG_OLD_INSTANCE",
    "is_scaling": False,
    "delete_publicip": False,
    "notifications": [
        "EMAIL"
    ]
}

EXAMPLE_EXTEND = {
    "networks": [
        {
            "id": " a8327883-6b07-4497-9c61-68d03ee193a "
        }
    ],
    "detail": None,
    "scaling_group_name": "healthCheck",
    "scaling_group_id": "77a7a397-7d2f-4e79-9da9-6a35e2709150",
    "scaling_group_status": "INSERVICE",
    "scaling_configuration_id": "1d281494-6085-4579-b817-c1f813be835f",
    "scaling_configuration_name": "healthCheck",
    "current_instance_number": 0,
    "desire_instance_number": 1,
    "min_instance_number": 0,
    "max_instance_number": 500,
    "cool_down_time": 300,
    "lbaas_listeners": [
        {
            "pool_id": "2f7dae72-fb59-4fa1-b663-042dcd030f81",
            "protocol_port": 80,
            "weight": 1
        }
    ],
    "security_groups": [
        {
            "id": "8a4b1d5b-0054-419f-84b1-5c8a59ebc829"
        }
    ],
    "create_time": "2015-07-23T02:46:29Z",
    "vpc_id": "863ccae2-ee85-4d27-bc5b-3ba2a198a9e2",
    "health_periodic_audit_method": "ELB_AUDIT",
    "health_periodic_audit_time": "5",
    "health_periodic_audit_grace_period": 600,
    "instance_terminate_policy": "OLD_CONFIG_OLD_INSTANCE",
    "is_scaling": False,
    "delete_publicip": False,
    "delete_volume": False,
    "notifications": [
        "EMAIL"
    ],
    "multi_az_priority_policy": "EQUILIBRIUM_DISTRIBUTE"
}


class TestGroup(base.TestCase):

    def setUp(self):
        super(TestGroup, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()
        self.sess.post = mock.Mock()
        self.sess.delete = mock.Mock()
        self.sess.put = mock.Mock()
        self.sess.get_project_id = mock.Mock()
        self.sot = group.Group(**EXAMPLE)

    def test_basic(self):
        sot = group.Group()
        self.assertEqual('scaling_group', sot.resource_key)
        self.assertEqual('scaling_groups', sot.resources_key)
        self.assertEqual('/scaling_group', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = group.Group(**EXAMPLE)
        self.assertEqual(EXAMPLE['scaling_group_id'], sot.id)
        self.assertEqual(EXAMPLE['scaling_group_name'], sot.name)
        self.assertEqual(EXAMPLE['create_time'], sot.create_time)

    def test_make_it_extend(self):
        sot = group.Group(**EXAMPLE_EXTEND)
        self.assertEqual(EXAMPLE_EXTEND['scaling_group_id'], sot.id)
        self.assertEqual(EXAMPLE_EXTEND['scaling_group_name'], sot.name)
        self.assertEqual(EXAMPLE_EXTEND['create_time'], sot.create_time)
        self.assertEqual(
            EXAMPLE_EXTEND['lbaas_listeners'], sot.lbaas_listeners
        )
        self.assertEqual(
            EXAMPLE_EXTEND['health_periodic_audit_grace_period'],
            sot.health_periodic_audit_grace_period
        )
        self.assertEqual(
            EXAMPLE_EXTEND['delete_volume'], sot.delete_volume
        )
        self.assertEqual(
            EXAMPLE_EXTEND['multi_az_priority_policy'],
            sot.multi_az_priority_policy
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.tests.functional.sdk.vlb import TestVlb


class TestFlavor(TestVlb):

    def setUp(self):
        super(TestFlavor, self).setUp()

    def test_list_flavors(self):
        flavor_name = 'L4_flavor.elb.s2.medium'
        flavors = list(self.client.flavors())
        self.assertGreaterEqual(len(flavors), 0)
        flavor = list(self.client.flavors(name=flavor_name))
        self.assertEqual(flavor[0].name, flavor_name)

    def test_get_flavor(self):
        flavor_name = 'L4_flavor.elb.s2.medium'
        flavors = list(self.client.flavors(name=flavor_name))
        flavor = self.client.get_flavor(flavors[0])
        self.assertEqual(flavor.name, flavor_name)

    def test_find_flavor(self):
        flavor_name = 'L4_flavor.elb.s2.medium'
        flavor = self.client.find_flavor(flavor_name)
        self.assertEqual(flavor.name, flavor_name)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from unittest.mock import call

import mock
from openstackclient.tests.unit import utils as tests_utils
from osc_lib import exceptions

from otcextensions.osclient.vpcep.v1 import endpoint
from otcextensions.tests.unit.osclient.vpcep.v1 import fakes


class TestListEndpoints(fakes.TestVpcep):

    objects = fakes.FakeEndpoint.create_multiple(3)

    column_list_headers = (
        'Id',
        'Endpoint Service Name',
        'Status',
        'Is Enabled',
    )

    columns = (
        'id',
        'endpoint_service_name',
        'status',
        'is_enabled',
    )

    data = []

    for s in objects:
        data.append(
            (
                s.id,
                s.endpoint_service_name,
                s.status,
                s.is_enabled,
            )
        )

    def setUp(self):
        super(TestListEndpoints, self).setUp()

        self.cmd = endpoint.ListEndpoints(self.app, None)

        self.client.endpoints = mock.Mock()
        self.client.api_mock = self.client.endpoints

    def test_list(self):
        arglist = []

        verifylist = []

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with()

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_args(self):
        arglist = [
            '--id',
            '1',
            '--service-name',
            '2',
            '--sort-key',
            'created_at',
            '--sort-dir',
            'desc',
            '--limit',
            '6',
            '--offset',
            '7',
        ]

        verifylist = [
            ('id', '1'),
            ('endpoint_service_name', '2'),
            ('sort_key', 'created_at'),
            ('sort_dir', 'desc'),
            ('limit', 6),
            ('offset', 7),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            id='1',
            endpoint_service_name='2',
            sort_key='created_at',
            sort_dir='desc',
            limit=6,
            offset=7,
        )


class TestCreateEndpoint(fakes.TestVpcep):

    _data = fakes.FakeEndpoint.create_one()

    columns = (
        'created_at',
        'endpoint_service_id',
        'endpoint_service_name',
        'id',
        'is_dns_enabled',
        'is_enabled',
        'is_whitelist_enabled',
        'marker_id',
        'project_id',
        'router_id',
        'service_type',
        'status',
        'tags',
        'updated_at',
        'whitelist',
    )

    data = fakes.gen_data(_data, columns, formatters=endpoint._formatters)

    def setUp(self):
        super(TestCreateEndpoint, self).setUp()

        self.cmd = endpoint.CreateEndpoint(self.app, None)
        self.client.create_endpoint = mock.Mock(
            return_value=fakes.FakeEndpoint.create_one()
        )

    def test_create(self):
        arglist = [
            '--service-id',
            '1',
            '--router-id',
            '2',
            '--network-id',
            '3',
            '--port-ip',
            '4',
            '--route-tables',
            'abc',
            '123',
            '--whitelist',
            'xyz',
            '456',
            '--specification-name',
            '5',
            '--description',
            '6',
            '--tags',
            'key=tag-key,value=tag-value',
            '--enable-dns',
            '--enable-whitelist',
        ]
        verifylist = [
            ('endpoint_service_id', '1'),
            ('vpc_id', '2'),
            ('subnet_id', '3'),
            ('port_ip', '4'),
            ('routetables', ['abc', '123']),
            ('whitelist', ['xyz', '456']),
            ('specification_name', '5'),
            ('description', '6'),
            ('tags', [{'key': 'tag-key', 'value': 'tag-value'}]),
            ('enable_dns', True),
            ('enable_whitelist', True),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        attrs = {
            'endpoint_service_id': '1',
            'vpc_id': '2',
            'subnet_id': '3',
            'port_ip': '4',
            'routetables': ['abc', '123'],
            'whitelist': ['xyz', '456'],
            'specification_name': '5',
            'description': '6',
            'tags': [{'key': 'tag-key', 'value': 'tag-value'}],
            'enable_dns': True,
            'enable_whitelist': True,
        }

        self.client.create_endpoint.assert_called_with(**attrs)
        self.assertEqual(self.columns, columns)


class TestShowEndpoint(fakes.TestVpcep):

    _data = fakes.FakeEndpoint.create_one()

    columns = (
        'created_at',
        'endpoint_service_id',
        'endpoint_service_name',
        'id',
        'is_dns_enabled',
        'is_enabled',
        'is_whitelist_enabled',
        'marker_id',
        'project_id',
        'router_id',
        'service_type',
        'status',
        'tags',
        'updated_at',
        'whitelist',
    )

    data = fakes.gen_data(_data, columns, formatters=endpoint._formatters)

    def setUp(self):
        super(TestShowEndpoint, self).setUp()

        self.cmd = endpoint.ShowEndpoint(self.app, None)

        self.client.get_endpoint = mock.Mock(return_value=self._data)

    def test_show_no_options(self):
        arglist = []
        verifylist = []

        # Testing that a call without the required argument will fail and
        # throw a "ParserExecption"
        self.assertRaises(
            tests_utils.ParserException,
            self.check_parser,
            self.cmd,
            arglist,
            verifylist,
        )

    def test_show(self):
        arglist = [
            self._data.id,
        ]

        verifylist = [
            ('endpoint', self._data.id),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        self.client.get_endpoint.assert_called_with(self._data.id)

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_show_non_existent(self):
        arglist = [
            'non-existing-endpoint-id',
        ]

        verifylist = [
            ('endpoint', 'non-existing-endpoint-id'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_result = exceptions.CommandError('Resource Not Found')
        self.client.find_service = mock.Mock(side_effect=find_mock_result)

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual('Resource Not Found', str(e))
        self.client.get_endpoint.assert_called_with('non-existing-endpoint-id')


class TestDeleteEndpoint(fakes.TestVpcep):

    _data = fakes.FakeEndpoint.create_multiple(2)

    def setUp(self):
        super(TestDeleteEndpoint, self).setUp()

        self.client.delete_endpoint = mock.Mock(return_value=None)

        # Get the command object to test
        self.cmd = endpoint.DeleteEndpoint(self.app, None)

    def test_delete(self):
        arglist = [
            self._data[0].id,
        ]

        verifylist = [
            ('endpoint', [self._data[0].id]),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.delete_endpoint.assert_called_with(self._data[0].id)
        self.assertIsNone(result)

    def test_multiple_delete(self):
        arglist = []

        for data in self._data:
            arglist.append(data.id)

        verifylist = [
            ('endpoint', arglist),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)

        calls = []
        for data in self._data:
            calls.append(call(data.id))
        self.client.delete_endpoint.assert_has_calls(calls)
        self.assertIsNone(result)

    def test_multiple_delete_with_exception(self):
        arglist = [
            self._data[0].id,
            'non-existing-endpoint-id',
        ]
        verifylist = [
            ('endpoint', arglist),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        delete_mock_result = [None, exceptions.CommandError]
        self.client.delete_endpoint = mock.Mock(side_effect=delete_mock_result)

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual(
                '1 of 2 VPC endpoint(s) failed to delete.', str(e)
            )

        self.client.delete_endpoint.assert_any_call(self._data[0].id)
        self.client.delete_endpoint.assert_any_call('non-existing-endpoint-id')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import uuid

import openstack
from openstack import _log

from otcextensions.tests.functional.sdk.mrs import TestMrs

_logger = _log.setup_logging('openstack')


class TestDS(TestMrs):
    data_source = None

    def setUp(self):
        super(TestDS, self).setUp()
        result = self.client.create_datasource(
            name=uuid.uuid4().hex,
            url='/simple/mapreduce/input',
            type='hdfs',
            description='test ds'
        )
        id = result.id
        self.data_source = self.client.get_datasource(id)

        _logger.debug("SAHARA create test ds")
        _logger.debug(self.data_source)

    def tearDown(self):
        try:
            if self.data_source.id:
                self.client.delete_datasource(self.data_source)
        except openstack.exceptions.SDKException as e:
            _logger.warning('Got exception during clearing resources %s'
                            % e.message)
        super(TestDS, self).tearDown()

    def test_01_list(self):
        self.data_sources = list(self.client.datasources())
        self.assertGreaterEqual(len(self.data_sources), 0)
        for ds in self.data_sources:
            _logger.debug(ds)

    # unstable test
    # def test_02_update(self):
    #     res = self.client.update_datasource(
    #         self.data_source,
    #         description='funct_test update ds'
    #     )
    #     _logger.debug(res)
    #     self.data_source = self.client.get_datasource(res)
    #     self.assertIsNotNone(self.data_source)
    #     self.assertEqual('funct_test update ds', res.description)
    #     _logger.debug(self.data_source)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
import mock

from otcextensions.osclient.cce.v1 import cluster_node
from otcextensions.tests.unit.osclient.cce.v1 import fakes


class TestClusterNode(fakes.TestCCE):

    def setUp(self):
        super(TestClusterNode, self).setUp()

    def test_flatten(self):
        _obj = fakes.FakeClusterNode.create_one()

        flat_data = cluster_node._flatten_cluster_node(_obj)

        data = (
            flat_data['id'],
            flat_data['name'],
            flat_data['cluster_uuid'],
            flat_data['flavor'],
            flat_data['cpu'],
            flat_data['memory'],
            flat_data['volumes'],
            flat_data['cpu_max'],
            flat_data['mem_max'],
            flat_data['pods_max'],
            flat_data['cpu_allocatable'],
            flat_data['mem_allocatable'],
            flat_data['pods_allocatable'],
            flat_data['conditions'],
            flat_data['private_ip'],
            flat_data['public_ip'],
            flat_data['availability_zone'],
            flat_data['ssh_key'],
            flat_data['status'],
            flat_data['replica_count'],
        )

        cmp_data = (
            _obj.metadata.id,
            _obj.metadata.name,
            _obj.spec.cluster_uuid,
            _obj.spec.flavor,
            _obj.spec.cpu,
            _obj.spec.memory,
            ';'.join(
                (x.disk_type + ':' + str(x.disk_size))
                for x in _obj.spec.volume
            ) if _obj.spec.volume else '',
            _obj.spec.status.capacity.cpu,
            _obj.spec.status.capacity.memory,
            _obj.spec.status.capacity.pods,
            _obj.spec.status.allocatable.cpu,
            _obj.spec.status.allocatable.memory,
            _obj.spec.status.allocatable.pods,
            ';'.join(
                (cond['type'] + '=' + cond['status'])
                for cond in _obj.spec.status.conditions
            ) if _obj.spec.status.conditions else '',
            _obj.spec.private_ip,
            _obj.spec.public_ip,
            _obj.spec.availability_zone,
            _obj.spec.ssh_key,
            _obj.status,
            _obj.replica_count
        )

        self.assertEqual(data, cmp_data)


class TestListClusterNode(fakes.TestCCE):

    _objs = fakes.FakeClusterNode.create_multiple(3)

    columns = ('ID', 'name', 'private_ip', 'public_ip',
               'availability_zone', 'status')

    data = []

    for s in _objs:
        flat_data = cluster_node._flatten_cluster_node(s)

        data.append((
            flat_data['id'],
            flat_data['name'],
            flat_data['private_ip'],
            flat_data['public_ip'],
            flat_data['availability_zone'],
            flat_data['status'],
        ))

    def setUp(self):
        super(TestListClusterNode, self).setUp()

        self.cmd = cluster_node.ListCCEClusterNode(self.app, None)

        self.client.cluster_nodes = mock.Mock()

    def test_list_default(self):
        arglist = ['cluster_id']

        verifylist = [
            ('cluster', 'cluster_id')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.cluster_nodes.side_effect = [
            self._objs
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.cluster_nodes.assert_called_once_with(
            'cluster_id'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestShowClusterNode(fakes.TestCCE):

    _obj = fakes.FakeClusterNode.create_one()

    columns = ('ID', 'name', 'cluster_uuid',
               'flavor', 'cpu', 'memory', 'volumes',
               'cpu_max', 'mem_max', 'pods_max',
               'cpu_allocatable', 'mem_allocatable', 'pods_allocatable',
               'conditions',
               'private_ip', 'public_ip', 'availability_zone', 'ssh_key',
               'status', 'replica_count')

    flat_data = cluster_node._flatten_cluster_node(_obj)

    data = (
        flat_data['id'],
        flat_data['name'],
        flat_data['cluster_uuid'],
        flat_data['flavor'],
        flat_data['cpu'],
        flat_data['memory'],
        flat_data['volumes'],
        flat_data['cpu_max'],
        flat_data['mem_max'],
        flat_data['pods_max'],
        flat_data['cpu_allocatable'],
        flat_data['mem_allocatable'],
        flat_data['pods_allocatable'],
        flat_data['conditions'],
        flat_data['private_ip'],
        flat_data['public_ip'],
        flat_data['availability_zone'],
        flat_data['ssh_key'],
        flat_data['status'],
        flat_data['replica_count'],
    )

    def setUp(self):
        super(TestShowClusterNode, self).setUp()

        self.cmd = cluster_node.ShowCCEClusterNode(self.app, None)

        self.client.find_cluster_node = mock.Mock()

    def test_show(self):
        arglist = [
            'cluster_uuid',
            'node_id'
        ]

        verifylist = [
            ('cluster', 'cluster_uuid'),
            ('node', 'node_id')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_cluster_node.side_effect = [
            self._obj
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_cluster_node.assert_called_once_with(
            cluster='cluster_uuid',
            node='node_id')

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestCreateClusterNode(fakes.TestCCE):

    _obj = fakes.FakeClusterNode.create_one()

    columns = ('ID', 'name', 'cluster_uuid',
               'flavor', 'cpu', 'memory', 'volumes',
               'cpu_max', 'mem_max', 'pods_max',
               'cpu_allocatable', 'mem_allocatable', 'pods_allocatable',
               'conditions',
               'private_ip', 'public_ip', 'availability_zone', 'ssh_key',
               'status', 'replica_count')

    flat_data = cluster_node._flatten_cluster_node(_obj)

    data = (
        flat_data['id'],
        flat_data['name'],
        flat_data['cluster_uuid'],
        flat_data['flavor'],
        flat_data['cpu'],
        flat_data['memory'],
        flat_data['volumes'],
        flat_data['cpu_max'],
        flat_data['mem_max'],
        flat_data['pods_max'],
        flat_data['cpu_allocatable'],
        flat_data['mem_allocatable'],
        flat_data['pods_allocatable'],
        flat_data['conditions'],
        flat_data['private_ip'],
        flat_data['public_ip'],
        flat_data['availability_zone'],
        flat_data['ssh_key'],
        flat_data['status'],
        flat_data['replica_count'],
    )

    def setUp(self):
        super(TestCreateClusterNode, self).setUp()

        self.cmd = cluster_node.CreateCCEClusterNode(self.app, None)

        self.client.add_node = mock.Mock()

    def test_create(self):
        arglist = [
            'cluster_name',
            '--flavor', 'flvr',
            '--label', 'lbl',
            '--volume', 'sys,sata,30',
            '--volume', 'data,sata,230',
            '--ssh_key', 'key',
            '--assign_floating_ip',
            '--availability_zone', 'az',
            '--tag', 'a=b',
            '--tag', 'c=d',
            '--replica_count', '12',
        ]

        verifylist = [
            ('cluster', 'cluster_name'),
            ('flavor', 'flvr'),
            ('label', 'lbl'),
            ('volume', ['sys,sata,30', 'data,sata,230']),
            ('ssh_key', 'key'),
            ('availability_zone', 'az'),
            ('assign_floating_ip', True),
            ('tag', ['a=b', 'c=d']),
            ('replica_count', 12)
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.add_node.side_effect = [
            self._obj
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.add_node.assert_called_once_with(
            cluster='cluster_name',
            replica_count=12,
            spec={
                'flavor': 'flvr',
                'ssh_key': 'key',
                'availability_zone': 'az',
                'label': 'lbl',
                'assign_floating_ip': True,
                'volume': [
                    {'disk_type': 'root', 'volume_type': 'SATA'},
                    {'disk_type': 'data', 'volume_type': 'SATA',
                        'disk_size': '230'}],
                'tags': ['a.b', 'c.d']}
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDeleteClusterNode(fakes.TestCCE):

    def setUp(self):
        super(TestDeleteClusterNode, self).setUp()

        self.cmd = cluster_node.DeleteCCEClusterNode(self.app, None)

        self.client.delete_cluster_nodes = mock.Mock()

    def test_delete(self):
        arglist = [
            'cluster_uuid',
            'node1',
            'node2'
        ]

        verifylist = [
            ('cluster', 'cluster_uuid'),
            ('node', ['node1', 'node2'])
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_cluster_nodes.side_effect = [{}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.delete_cluster_nodes.assert_called_once_with(
            cluster='cluster_uuid',
            node_names=['node1', 'node2']
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack import resource


class SearchPropSpec(resource.Resource):
    #: Relationship between attribute values.
    op = resource.Body("op")
    #: Check the document. Search criteria of an attribute.
    props = resource.Body("props", type=dict)


class SearchLabelSpec(resource.Resource):
    #: Label name.
    name = resource.Body("name")
    #: Operation type between multiple attributes.
    op = resource.Body("op")
    #: Label attribute, which is in the Object format and
    #:  stores any key-value pairs.
    property = resource.Body("property", type=dict)
    #: Label type.
    type = resource.Body("type", type=int)


class SearchLabelsSpec(resource.Resource):
    #: List of label search criteria.
    labels = resource.Body("labels", type=list, list_type=SearchLabelSpec)
    #: If you want to search for multiple labels, op must be specified.
    op = resource.Body("op")


class SearchConditionSpec(resource.Resource):
    #: Filter by coefficient of difficulty.
    coefficient = resource.Body("coefficient")
    #: A frame in the video.
    frame_in_video = resource.Body("frame_in_video", type=int)
    #: Whether a sample is a hard sample.
    hard = resource.Body("hard")
    #: Filter by data source.
    import_origin = resource.Body("import_origin")
    #: CT dosage, filtered by dosage.
    kvp = resource.Body("kvp")
    #: Label search criteria.
    label_list = resource.Body("label_list", type=SearchLabelsSpec)
    #: Labeler.
    labeler = resource.Body("labeler")
    #: Search by sample attribute.
    metadata = resource.Body("metadata", type=SearchPropSpec)
    #: Parent sample ID.
    parent_sample_id = resource.Body("parent_sample_id")
    #: Directory where data samples are stored (the directory must end with a
    #:  slash (/)).
    sample_dir = resource.Body("sample_dir")
    #: Search by sample name, including the file name extension.
    sample_name = resource.Body("sample_name")
    #: When a sample is added to the dataset, an index is created based on the
    #:  last modification time (accurate to day) of the sample on OBS.
    sample_time = resource.Body("sample_time")
    #: Search by confidence.
    score = resource.Body("score")
    #: DICOM layer thickness.
    slice_thickness = resource.Body("slice_thickness")
    #: DICOM scanning time.
    study_date = resource.Body("study_date")
    #: A time point in the video.
    time_in_video = resource.Body("time_in_video")


class ExportParamsSpec(resource.Resource):
    #: Whether to clear difficult.
    clear_difficult = resource.Body("clear_difficult", type=bool)
    #: Whether to clear hard example attributes.
    clear_hard_property = resource.Body("clear_hard_property", type=bool)
    #: Format of the dataset version to which data is exported.
    export_dataset_version_format = resource.Body(
        "export_dataset_version_format"
    )
    #: Name of the dataset version to which data is exported.
    export_dataset_version_name = resource.Body("export_dataset_version_name")
    #: Export destination.
    export_dest = resource.Body("export_dest")
    #: Name of the new dataset to which data is exported.
    export_new_dataset_name = resource.Body("export_new_dataset_name")
    #: Working directory of the new dataset to which data is exported.
    export_new_dataset_work_path = resource.Body(
        "export_new_dataset_work_path"
    )
    #: Whether to randomly allocate the training set and validation set based
    #:  on the specified ratio.
    ratio_sample_usage = resource.Body("ratio_sample_usage", type=bool)
    #: Sample status.
    sample_state = resource.Body("sample_state")
    #: ID list of exported samples.
    samples = resource.Body("samples", type=list)
    #: Exported search conditions.
    search_conditions = resource.Body(
        "search_conditions", type=list, list_type=SearchConditionSpec
    )
    #: Split ratio of training set and verification set during specified
    #:  version release.
    train_sample_ratio = resource.Body("train_sample_ratio")


class DatasetExportTask(resource.Resource):
    base_path = "/datasets/%(uri_dataset_id)s/export-tasks"

    resources_key = "export_tasks"

    _query_mapping = resource.QueryParameters(
        "export_type",
        "limit",
        "offset",
    )

    # capabilities
    allow_create = True
    allow_list = True
    allow_fetch = True

    # Properties
    #: Labeling format.
    annotation_format = resource.Body("annotation_format")
    #: Time when a task is created.
    created_at = resource.Body("create_time", type=int)
    #: Dataset ID.
    dataset_id = resource.Body("dataset_id")
    #: Dataset ID.
    uri_dataset_id = resource.URI("uri_dataset_id")
    #: Dataset type.
    dataset_type = resource.Body("dataset_type", type=int)
    #: Error code.
    error_code = resource.Body("error_code")
    #: Error message.
    error_msg = resource.Body("error_msg")
    #: Format of the exported directory.
    export_format = resource.Body("export_format", type=int)
    #: Parameters of a dataset export task.
    export_params = resource.Body("export_params", type=ExportParamsSpec)
    #: Export type.
    export_type = resource.Body("export_type", type=int)
    #: Number of completed samples.
    finished_sample_count = resource.Body("finished_sample_count", type=int)
    #: Export output path.
    path = resource.Body("path")
    #: Percentage of current task progress.
    progress = resource.Body("progress", type=float)
    #: Sample status.
    sample_state = resource.Body("sample_state")
    #: Prefix of the OBS path in the exported labeling file.
    source_type_header = resource.Body("source_type_header")
    #: Task status.
    status = resource.Body("status", type=int)
    #: Task ID.
    task_id = resource.Body("task_id", alternate_id=True)
    #: Total number of export tasks.
    total_count = resource.Body("total_count", type=int)
    #: Total number of samples.
    total_sample_count = resource.Body("total_sample_count", type=int)
    #: Time when a task is updated.
    updated_at = resource.Body("update_time", type=int)
    #: Format of a dataset version.
    version_format = resource.Body("version_format")
    #: Dataset version ID.
    version_id = resource.Body("version_id")
    #: Whether to write the column name in the first line of the CSV file
    #:  during export.
    with_column_header = resource.Body("with_column_header", type=bool)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""Flavor v1 action implementations"""
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def set_attributes_for_print_detail(instance):
    info = {}  # instance.copy()
    info['id'] = instance.id
    info['ram'] = instance.ram
    info['name'] = instance.name
    # info['str_id'] = instance['str_id']
    if getattr(instance, 'flavor_detail', None):
        for det in instance.flavor_detail:
            if det['name'] == 'cpu':
                info['vcpus'] = det['value']
    return info


class ListDatabaseFlavors(command.Lister):

    _description = _("List database flavors")
    columns = ['ID', 'Name', 'ram', 'spec_code']

    def get_parser(self, prog_name):
        parser = super(ListDatabaseFlavors, self).get_parser(prog_name)

        parser.add_argument(
            'dbId',
            metavar='<dbId>',
            help=_('ID of the datastore version.'),
        )

        parser.add_argument(
            'region',
            metavar='<region>',
            default='eu-de',
            nargs='?',
            help=_('Region. `eu-de` is left empty'),
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds

        data = client.flavors(dbId=parsed_args.dbId, region=parsed_args.region)

        return (
            self.columns,
            (utils.get_item_properties(
                s,
                self.columns,
            ) for s in data)
        )


class ShowDatabaseFlavor(command.ShowOne):
    _description = _("Shows details of a database flavor")
    columns = ('ID', 'Name', 'ram', 'spec_code')

    def get_parser(self, prog_name):
        parser = super(ShowDatabaseFlavor, self).get_parser(prog_name)
        parser.add_argument(
            'flavor',
            metavar='<flavor>',
            help=_('ID or name of the flavor'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds

        obj = client.get_flavor(parsed_args.flavor)

        return (
            self.columns,
            utils.get_item_properties(
                obj,
                self.columns,
            )
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""DIS Stream v2 action implementations"""
import logging

from osc_lib import utils
from osc_lib.cli import parseractions
from osc_lib import exceptions
from osc_lib.command import command

from otcextensions.i18n import _
from otcextensions.common import sdk_utils
from otcextensions.common import cli_utils


LOG = logging.getLogger(__name__)


_formatters = {
    'partitions': cli_utils.YamlFormat,
    'created_at': cli_utils.UnixTimestampFormatter,
    'updated_at': cli_utils.UnixTimestampFormatter,
}


def _get_columns(item):
    column_map = {}
    hidden = [
        'location',
    ]
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map,
                                                           hidden)


STREAM_TYPE_CHOICES = ('COMMON', 'ADVANCED',)
DATA_TYPE_CHOICES = ('BLOB',)
COMPRESSION_FORMAT_CHOICES = ('snappy', 'gzip', 'zip',)


class ListStreams(command.Lister):

    _description = _("List Dis Streams.")
    display_columns = (
        'Name',
        'Stream Type',
        'Data Type',
        'Partition Count',
        'AutoScale Enabled',
        'Status'
    )
    columns = (
        'name',
        'stream_type',
        'data_type',
        'partition_count',
        'is_auto_scale_enabled',
        'status'
    )

    def get_parser(self, prog_name):
        parser = super(ListStreams, self).get_parser(prog_name)

        parser.add_argument(
            '--limit',
            metavar='<limit>',
            type=int,
            help=_("Limit to fetch number of records."),
        )
        parser.add_argument(
            '--start-stream-name',
            metavar='<start_stream_name>',
            help=_("Limit to fetch number of records."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dis
        args_list = (
            'limit',
            'start_stream_name',
        )
        attrs = {}
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val

        data = client.streams(**attrs)

        return (
            self.display_columns,
            (utils.get_item_properties(s, self.columns) for s in data)
        )


class ShowStream(command.ShowOne):
    _description = _("Show DIS Stream details.")

    def get_parser(self, prog_name):
        parser = super(ShowStream, self).get_parser(prog_name)
        parser.add_argument(
            'streamName',
            metavar='<streamName>',
            help=_("Specifies the name of the DIS Stream."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dis
        obj = client.get_stream(parsed_args.streamName)
        obj.id = obj.stream_id

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)

        return (display_columns, data)


class CreateStream(command.ShowOne):
    _description = _("Create new DIS Stream.")

    def get_parser(self, prog_name):
        parser = super(CreateStream, self).get_parser(prog_name)
        parser.add_argument(
            'streamName',
            metavar='<streamName>',
            help=_("Specify the name of the DIS Stream."),
        )
        parser.add_argument(
            '--partition-count',
            metavar='<partition_count>',
            type=int,
            required=True,
            help=_("Number of partitions. Partitions are the base throughput "
                   "unit of the DIS stream."),
        )

        parser.add_argument(
            '--stream-type',
            dest='stream_type',
            metavar='{' + ','.join(STREAM_TYPE_CHOICES) + '}',
            type=lambda s: s.upper(),
            choices=STREAM_TYPE_CHOICES,
            help=_("Stream type. Supported Types:"
                   "\nCOMMON: a common stream with a bandwidth of 1 MB/s."
                   "\nADVANCED: an advanced stream with a bandwidth of 5 MB/s."
                   "\nDefault value: COMMON."),
        )
        parser.add_argument(
            '--data-type',
            dest='data_type',
            metavar='{' + ','.join(DATA_TYPE_CHOICES) + '}',
            type=lambda s: s.upper(),
            choices=DATA_TYPE_CHOICES,
            help=_("Source data type. Supported Types:"
                   "\nBLOB: a collection of binary data stored as a single "
                   "entity in a database management system."
                   "\nDefault value: BLOB."),
        )
        parser.add_argument(
            '--data-duration',
            metavar='<data_duration>',
            type=int,
            help=_("Data retention period."
                   "\nValue range: 24–72"
                   "\nUnit: hour"
                   "\nDefault value: 24"),
        )
        parser.add_argument(
            '--autoscale',
            action='store_true',
            help=_("Whether to enable auto scaling."),
        )
        parser.add_argument(
            '--autoscale-min-count',
            metavar='<autoscale_min_count>',
            dest='auto_scale_min_partition_count',
            type=int,
            default=1,
            help=_("Minimum number of partitions for automatic scale-down "
                   "when auto scaling is enabled. Default: 1"),
        )
        parser.add_argument(
            '--autoscale-max-count',
            metavar='<autoscale_max_count>',
            dest='auto_scale_max_partition_count',
            type=int,
            default=1,
            help=_("Maximum number of partitions for automatic scale-up when "
                   "auto scaling is enabled. Default: 1"),
        )
        parser.add_argument(
            '--compression-format',
            dest='compression_format',
            metavar='{' + ','.join(COMPRESSION_FORMAT_CHOICES) + '}',
            type=lambda s: s.lower(),
            choices=COMPRESSION_FORMAT_CHOICES,
            help=_("Data compression type. The following types are available:"
                    "\nsnappy"
                    "\ngzip"
                    "\nzip"
                    "\nData is not compressed by default."),
        )
        parser.add_argument(
            '--tag',
            action=parseractions.MultiKeyValueAction,
            metavar='key=<key>,value=<value>',
            required_keys=['key', 'value'],
            dest='tags',
            help=_('Add Tag(s) to a Stream.\n'
                   'key=<key>: Tag key. The value can contain 1 to 36 '
                   'characters. Only digits, letters, hyphens (-) and '
                   'underscores (_) are allowed.\n'
                   'value=<value>: Tag value. The value can contain 0 to 43 '
                   'characters. Only digits, letters, hyphens (-) and '
                   'underscores (_) are allowed.'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dis

        args_list = (
            'partition_count',
            'stream_type',
            'data_type',
            'data_duration',
            'auto_scale_min_partition_count',
            'auto_scale_max_partition_count',
            'compression_format',
            'tags',
        )
        attrs = {
            'name': parsed_args.streamName
        }
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val

        if parsed_args.autoscale:
            attrs.update(auto_scale_enabled=True)

        obj = client.create_stream(**attrs)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class UpdateStreamPartition(command.ShowOne):
    _description = _("Update Partition Quantity of a DIS Stream.")

    columns = (
        'name',
        'current_partition_count',
        'target_partition_count',
    )

    display_columns = (
        'stream_name',
        'current_partition_count',
        'target_partition_count',
    )

    def get_parser(self, prog_name):
        parser = super(UpdateStreamPartition, self).get_parser(prog_name)
        parser.add_argument(
            'streamName',
            metavar='<streamName>',
            help=_("Specifies the Name of the DIS Stream."),
        )
        parser.add_argument(
            '--partition-count',
            metavar='<partition_count>',
            type=int,
            required=True,
            help=_("Number of the target partitions.The value is an integer "
                   "greater than 0."
                   "\nEach stream can be scaled up and down for five times "
                   "within one hour. After a stream is scaled up or down, it "
                   "cannot be scaled up or down again in the next one hour."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dis
        obj = client.update_stream_partition(
            parsed_args.streamName, parsed_args.partition_count)

        data = utils.get_item_properties(obj, self.columns)

        return (self.display_columns, data)


class DeleteStream(command.Command):

    _description = _("Deletes DIS Stream.")

    def get_parser(self, prog_name):
        parser = super(DeleteStream, self).get_parser(prog_name)
        parser.add_argument(
            'streamName',
            metavar='<streamName>',
            nargs='+',
            help=_("Name of Dis Stream(s) to delete."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dis
        result = 0
        for stream_name in parsed_args.streamName:
            try:
                client.delete_stream(stream_name)
            except Exception as e:
                result += 1
                LOG.error(_("Failed to delete Dis Stream with "
                          "name '%(stream_name)s': %(e)s"),
                          {'stream_name': stream_name, 'e': e})
        if result > 0:
            total = len(parsed_args.streamName)
            msg = (_("%(result)s of %(total)s DIS Stream(s) failed "
                   "to delete.") % {'result': result, 'total': total})
            raise exceptions.CommandError(msg)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from otcextensions.tests.functional.sdk.dds import TestDds


class TestDatastores(TestDds):
    datastore_name = 'DDS-Community'

    def setUp(self):
        super(TestDatastores, self).setUp()

    def test_list_datastores(self):
        datastores = list(self.client.datastores(
            datastore_name=self.datastore_name))
        self.assertIsNotNone(datastores)

    def test_list_datastore_types(self):
        datastore_types = list(self.client.datastore_types())
        self.assertEqual('DDS-Community', datastore_types[0].name)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class Gateway(resource.Resource):
    resources_key = 'nat_gateways'
    resource_key = 'nat_gateway'
    base_path = '/nat_gateways'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'admin_state_up', 'created_at', 'description', 'id',
        'internal_network_id', 'limit', 'name', 'router_id',
        'spec', 'status', 'project_id', project_id='tenant_id'
    )

    # Properties
    #: Specifies whether GW is up or down
    #: *true:* Gw is up
    #: *false:* GW is down
    admin_state_up = resource.Body('admin_state_up', type=bool)
    #: Specifies when GW was is created
    #: format is *yyyy-mm-dd hh:mm:ss*
    created_at = resource.Body('created_at')
    #: Provides description of gateway
    description = resource.Body('description')
    #: Specifies the ID of the gateway.
    id = resource.Body('id')
    #: Specifies the network ID of the downstream interface
    internal_network_id = resource.Body('internal_network_id')
    #: Specifies the name of the gateway.
    #: Contains only digits, letters, underscores and hyphens
    name = resource.Body('name')
    #: Specifies the project ID
    project_id = resource.Body('tenant_id')
    #: Specifies the router ID
    router_id = resource.Body('router_id')
    #: Specifies the type of the gateway.
    #: *1:* small type, supports up to 10,000 SNAT connections
    #: *2:* medium type, supports up to 50,000 SNAT connections
    #: *3:* large type, supports up to 200,000 SNAT connections
    #: *4:* extra-large type, supports up to 1,000,000 SNAT connections
    spec = resource.Body('spec')
    #: Specifies the status
    status = resource.Body('status')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from otcextensions.tests.functional.sdk.apig import TestApiG


class TestCustomAuthorizer(TestApiG):
    custom_authorizer = None
    gateway = '560de602c9f74969a05ff01d401a53ed'

    def setUp(self):
        super(TestCustomAuthorizer, self).setUp()
        attrs = {
            'name': 'custom_auth_test',
            'type': 'BACKEND',
            'authorizer_type': 'FUNC',
            'authorizer_uri': 'urn:fss:eu-de:7ed5f793b8354ea9b27a849f17af4733'
                              ':function:default:test_apig_authorizer:latest',
            'authorizer_version': '1'
        }
        result = self.client.create_custom_authorizer(
            gateway=TestCustomAuthorizer.gateway,
            **attrs)
        self.assertIsNotNone(result.id)
        TestCustomAuthorizer.custom_authorizer = result

    def tearDown(self):
        super(TestCustomAuthorizer, self).tearDown()
        self.client.delete_custom_authorizer(
            gateway=TestCustomAuthorizer.gateway,
            custom_authorizer=TestCustomAuthorizer.custom_authorizer.id
        )
        found = list(self.client.custom_authorizers(
            gateway=TestCustomAuthorizer.gateway,
        ))
        self.assertEqual(len(found), 0)

    def test_02_get_custom_authorizer(self):
        found = self.client.get_custom_authorizer(
            gateway=TestCustomAuthorizer.gateway,
            custom_authorizer=TestCustomAuthorizer.custom_authorizer.id)
        self.assertEqual(TestCustomAuthorizer.custom_authorizer.id, found.id)

    def test_03_list_custom_authorizer(self):
        attrs = {
            'limit': 2
        }
        found = list(self.client.custom_authorizers(
            gateway=TestCustomAuthorizer.gateway,
            **attrs
        ))
        self.assertEqual(len(found), 1)

    def test_04_update_custom_authorizer(self):
        change = 'BACKEND'
        attrs = {
            'name': 'custom_auth_test',
            'type': change,
            'authorizer_type': 'FUNC',
            'authorizer_uri': 'urn:fss:eu-de:7ed5f793b8354ea9b27a849f17af4733'
                              ':function:default:test_apig_authorizer:latest',
            'authorizer_version': '1',
            'ttl': 5,
            "identities": [{
                "name": "header",
                "location": "HEADER"
            }]
        }
        updated = self.client.update_custom_authorizer(
            gateway=TestCustomAuthorizer.gateway,
            custom_authorizer=TestCustomAuthorizer.custom_authorizer.id,
            **attrs
        )
        self.assertEqual(updated.type, change)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
# import six
from openstack import exceptions
from openstack import resource


class Stream(resource.Resource):
    base_path = '/streams'

    resources_key = 'stream_info_list'

    allow_create = True
    allow_list = True
    allow_commit = True
    allow_delete = True
    allow_fetch = True
    allow_patch = True

    _query_mapping = resource.QueryParameters('limit', 'start_stream_name')

    # Properties
    #: Minimum number of partitions for automatic scale-down when
    #:  auto scaling is enabled.
    auto_scale_min_partition_count = \
        resource.Body('auto_scale_min_partition_count', type=int)
    #: Maximum number of partitions for automatic scale-up when
    #:  auto scaling is enabled.
    auto_scale_max_partition_count = \
        resource.Body('auto_scale_max_partition_count', type=int)
    #: Data compression type. The following types are available:
    #: \nsnappy
    #: \ngzip
    #: \nzip
    #: \nData is not compressed by default.
    compression_format = resource.Body('compression_format')
    #: Timestamp at which the DIS stream was created.
    created_at = resource.Body('create_time', type=int)
    #: Partition Count of a stream before updating.
    current_partition_count = \
        resource.Body('current_partition_count', type=int)
    #: Data retention period.
    #: \nValue range: 24–72
    #: \nUnit: hour
    #: \nDefault value: 24
    data_duration = resource.Body('data_duration', type=int)
    #: Source data type.
    data_type = resource.Body('data_type', type=str)
    #: Specifies whether there are more matching partitions
    #:  of the DIS stream to list.
    has_more_partitions = resource.Body('has_more_partitions', type=bool)
    #: Specifies whether to enable auto scaling.
    #: \n`true`: auto scaling is enabled.
    #: \n`false`: auto scaling is disabled.
    is_auto_scale_enabled = resource.Body('auto_scale_enabled', type=bool)
    #: Stream name.
    name = resource.Body('stream_name', alias='stream_name')
    #: Number of partitions.
    #:  Partitions are the base throughput unit of the DIS stream.
    partition_count = resource.Body('partition_count', type=int)
    #: A list of partitions that comprise the DIS stream.
    partitions = resource.Body('partitions', type=list, list_type=dict)
    #: Total number of readable partitions (including
    #:  partitions in ACTIVE and DELETED state).
    readable_partition_count = \
        resource.Body('readable_partition_count', type=int)
    #: Period for storing data in units of hours.
    retention_period = resource.Body('retention_period', type=int)
    #: Current status of the stream. Possible values:
    #: \n`CREATING`: The stream is being created.
    #: \n`RUNNING`: The stream is running.
    #: \n`TERMINATING`: The stream is being deleted.
    #: \n`TERMINATED`: The stream has been deleted.
    status = resource.Body('status')
    #: Stream Id.
    stream_id = resource.Body('stream_id', alternate_id=True)
    #: Stream type.
    #: \n`COMMON`: a common stream. The bandwidth is 1 MB/s.
    #: \n`ADVANCED`: an advanced stream. The bandwidth is 5 MB/s.
    stream_type = resource.Body('stream_type')
    #: Enterprise project of a stream.
    sys_tags = resource.Body('sys_tags', type=list, list_type=dict)
    #: List of stream tags.
    tags = resource.Body('tags', type=list, list_type=dict)
    #: Number of the target partitions. The value is an integer greater than 0.
    target_partition_count = resource.Body('target_partition_count', type=int)
    #: Timestamp at which the DIS stream was most recently modified.
    updated_at = resource.Body('last_modified_time', type=int)
    #: Total number of writable partitions (including
    #:  partitions in ACTIVE state only).
    writable_partition_count = \
        resource.Body('writable_partition_count', type=int)

    @classmethod
    def list(cls, session, paginated=True, base_path=None,
             allow_unknown_params=False, **params):

        if not cls.allow_list:
            raise exceptions.MethodNotSupported(cls, "list")
        session = cls._get_session(session)
        microversion = cls._get_microversion(session)

        if base_path is None:
            base_path = cls.base_path
        params = cls._query_mapping._validate(
            params, base_path=base_path,
            allow_unknown_params=allow_unknown_params)

        query_params = cls._query_mapping._transpose(params, cls)
        uri = base_path % params

        limit = query_params.get('limit')

        while uri:
            # Copy query_params due to weird mock unittest interactions
            response = session.get(
                uri,
                headers={"Accept": "application/json"},
                params=query_params.copy(),
                microversion=microversion)
            exceptions.raise_from_response(response)
            data = response.json()
            # Discard any existing pagination keys
            query_params.pop('start_stream_name', None)
            query_params.pop('limit', None)

            if cls.resources_key:
                resources = data[cls.resources_key]
            else:
                resources = data

            if not isinstance(resources, list):
                resources = [resources]

            for raw_resource in resources:
                value = cls.existing(
                    microversion=microversion,
                    connection=session._get_connection(),
                    **raw_resource)
                yield value

            if resources and paginated:
                uri, next_params = cls._get_next_link(
                    uri, response, data, limit)
                query_params.update(next_params)
            else:
                return

    @classmethod
    def _get_next_link(cls, uri, response, data, limit):
        # DIS service pagination. Returns query for the next page
        next_link = None
        params = {}
        if 'has_more_streams' in data and data['has_more_streams']:
            next_link = uri
            params['start_stream_name'] = data['stream_names'][-1]
            params['limit'] = limit
        else:
            next_link = None
        query_params = cls._query_mapping._transpose(params, cls)
        return next_link, query_params



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import _log

from otcextensions.tests.functional import base

_logger = _log.setup_logging('openstack')


class TestPredefinedTag(base.BaseFunctionalTest):

    def setUp(self):
        super(TestPredefinedTag, self).setUp()
        self.tms = self.conn.tms

    def test_create_tag(self):
        attrs = {
            'key': 'key2',
            'value': 'value2'
        }
        self.tms.create_predefined_tag(**attrs)
        attrs = {
            'key': 'key1',
            'value': 'value1'
        }
        self.tms.create_predefined_tag(**attrs)

    def test_update_tag(self):
        attrs = {
            "new_tag": {
                "key": "check_this",
                "value": "mate"},
            "old_tag": {
                'key': 'key1',
                'value': 'value1'
            }
        }
        self.tms.update_predefined_tag(**attrs)

    def test_delete_tag(self):
        attrs = {
            'key': 'key2',
            'value': 'value2'
        }
        self.tms.delete_predefined_tag(**attrs)

    def test_tags_list(self):
        tags = list(self.tms.predefined_tags())
        self.assertGreaterEqual(len(tags), 0)

    def test_clear_tags(self):
        attrs = {
            "key": "check_this",
            "value": "mate"
        }
        self.tms.delete_predefined_tag(**attrs)
        attrs = {
            'key': 'key1',
            'value': 'value1'
        }
        self.tms.delete_predefined_tag(**attrs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource

from otcextensions.sdk.identity.v3 import _bad_base as _base


class AgencyRole(_base.BadBaseResource):
    resources_key = 'roles'
    base_path = ('/v3.0/OS-AGENCY/%(role_ref_type)ss/%(role_ref_id)s'
                 '/agencies/%(agency_id)s/roles')

    # capabilities
    allow_commit = True
    allow_head = True
    allow_delete = True
    allow_list = True

    # Properties
    role_ref_type = resource.URI('role_ref_type')
    role_ref_id = resource.URI('role_ref_id')
    agency_id = resource.URI('agency_id')

    #: Directory where a role locates.
    catalog = resource.Body('catalog')
    #: Description of the role.
    description = resource.Body('description')
    #: ID of the domain to which a role belongs.
    domain_id = resource.Body('domain_id')
    #: Name of a role.
    name = resource.Body('display_name')
    #: Policy of a role.
    policy = resource.Body('policy', type=dict)
    #: Display mode of a role.
    type = resource.Body('type')



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json

from otcextensions.tests.functional.osclient.dcaas.v2 import common


class TestEndpointGroup(common.DcaasTestCase):

    def setUp(self):
        super(TestEndpointGroup, self).setUp()

    def test_list_endpoint_groups(self):
        json_output = json.loads(self.openstack(
            'dcaas endpoint group list -f json '
        ))
        self.assertIsNotNone(json_output)

    def test_find_endpoint_group(self):
        json_output = json.loads(self.openstack(
            'dcaas endpoint group show {id} -f json'.format(
                id=self.EP_GROUP_ID
            )
        ))
        self.assertIsNotNone(json_output)
        self.assertEqual(json_output['name'], self.EP_GROUP_NAME)

    def test_update_endpoint_group(self):
        ep_group_name = self.EP_GROUP_NAME + '-updated'
        json_output = json.loads(self.openstack(
            'dcaas endpoint group update {id} --name {name} -f json'.format(
                id=self.EP_GROUP_ID,
                name=ep_group_name
            )
        ))
        self.assertEqual(json_output['id'], self.EP_GROUP_ID)
        self.assertEqual(json_output['name'], ep_group_name)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from cryptography import x509
from cryptography.x509.oid import NameOID
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import rsa
import datetime


class SelfSignedCertificateGenerator:
    def __init__(self, domain_name, valid_days=365):
        """
        Initialize with the common name (domain or identifier)
         and validity period.
        """
        self.domain_name = domain_name
        self.valid_days = valid_days
        self.private_key = None
        self.certificate = None

    def generate_private_key(self, key_size=2048):
        """
        Generates an RSA private key.
        """
        self.private_key = rsa.generate_private_key(
            public_exponent=65537,
            key_size=key_size,
            backend=default_backend()
        )
        return self.private_key

    def generate_certificate(self):
        """
        Generates a self-signed certificate using the generated private key.
        """
        if self.private_key is None:
            raise ValueError(
                "Generate a private key first using generate_private_key()."
            )

        subject = issuer = x509.Name([
            x509.NameAttribute(NameOID.COMMON_NAME, self.domain_name),
        ])

        self.certificate = x509.CertificateBuilder().subject_name(
            subject
        ).issuer_name(
            issuer
        ).public_key(
            self.private_key.public_key()
        ).serial_number(
            x509.random_serial_number()
        ).not_valid_before(
            datetime.datetime.now(datetime.UTC) - datetime.timedelta(days=1)
        ).not_valid_after(
            datetime.datetime.now(datetime.UTC) + datetime.timedelta(
                days=self.valid_days)
        ).add_extension(
            x509.SubjectAlternativeName([x509.DNSName(self.domain_name)]),
            critical=False,
        ).sign(self.private_key, hashes.SHA256(), default_backend())

        return self.certificate

    def save_private_key(self, filename, password=None):
        """
        Saves the private key to a file. If a password is provided,
         the key will be encrypted.
        """
        if self.private_key is None:
            raise ValueError("Private key not generated yet.")

        encryption = (serialization.NoEncryption() if password is None
                      else serialization.BestAvailableEncryption(
            password.encode()))

        with open(filename, "wb") as key_file:
            key_file.write(self.private_key.private_bytes(
                encoding=serialization.Encoding.PEM,
                format=serialization.PrivateFormat.TraditionalOpenSSL,
                encryption_algorithm=encryption
            ))
        print(f"Private key saved to {filename}")

    def save_certificate(self, filename):
        """
        Saves the certificate to a file.
        """
        if self.certificate is None:
            raise ValueError("Certificate not generated yet.")

        with open(filename, "wb") as cert_file:
            cert_file.write(self.certificate.public_bytes(
                serialization.Encoding.PEM))
        print(f"Certificate saved to {filename}")

    def get_private_key(self):
        """
        Gets the private.
        """
        if self.private_key is None:
            raise ValueError("Private key not generated yet.")

        key_bytes = self.private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.TraditionalOpenSSL,
            encryption_algorithm=serialization.NoEncryption()
        )
        return key_bytes.decode('utf-8')

    def get_certificate(self):
        """
        Gets the certificate.
        """
        if self.certificate is None:
            raise ValueError("Certificate not generated yet.")

        key_bytes = self.certificate.public_bytes(
            serialization.Encoding.PEM)
        return key_bytes.decode('utf-8')



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import random
import uuid

import mock

from otcextensions.sdk.cce.v1 import cluster
from otcextensions.sdk.cce.v1 import cluster_node
from otcextensions.tests.unit.osclient import test_base


class TestCCE(test_base.TestCommand):

    def setUp(self):
        super(TestCCE, self).setUp()

        self.app.client_manager.cce = mock.Mock()
        self.client = self.app.client_manager.cce


class FakeCluster(test_base.Fake):
    """Fake one or more Cluster"""

    @classmethod
    def generate(cls, cnt_hosts=1):
        hosts = []
        for i in range(cnt_hosts - 1):
            host = {'kind': 'host'}
            hosts.append(host)
        object_info = {
            'kind': 'cluster',
            'metadata': {
                'uuid': 'id-' + uuid.uuid4().hex,
                'name': 'name-' + uuid.uuid4().hex,
            },
            'spec': {
                'cpu': random.randint(1, 15),
                'memory': random.randint(1, 150000),
                'endpoint': 'endpoint-' + uuid.uuid4().hex,
                'availability_zone': 'az-' + uuid.uuid4().hex,
                'vpc': 'vpc-' + uuid.uuid4().hex,
                'hostList': {
                    'spec': {
                        'hostList': hosts
                    }
                }
            },
            'status': {
                'status': 'sts'
            }
        }
        obj = cluster.Cluster.existing(**object_info)
        return obj

    @classmethod
    def create_one(cls, cnt_hosts=1):
        """Create a fake resource.
        """
        resource = cls.generate(cnt_hosts)

        return resource

    @classmethod
    def create_multiple(cls, count=2, attrs=None):
        """Create multiple fake resources.
        """
        objects = []
        for i in range(0, count):
            objects.append(
                cls.create_one(1))

        return objects


class FakeClusterNode(test_base.Fake):
    """Fake one or more Cluster node"""

    @classmethod
    def generate(cls):
        object_info = {
            'kind': 'host',
            'metadata': {
                'uuid': 'id-' + uuid.uuid4().hex,
                'name': 'name-' + uuid.uuid4().hex,
            },
            'spec': {
                'cluster_uuid': 'clid-' + uuid.uuid4().hex,
                'cpu': random.randint(1, 15),
                'memory': random.randint(1, 150000),
                'availability_zone': 'az-' + uuid.uuid4().hex,
                'flavor': 'flavor-' + uuid.uuid4().hex,
                'public_ip': 'public_ip-' + uuid.uuid4().hex,
                'private_ip': 'private_ip-' + uuid.uuid4().hex,
                'volume': [
                    {
                        'disk_type': 'dt' + uuid.uuid4().hex,
                        'disk_size': random.randint(1, 15000),
                    },
                    {
                        'disk_type': 'dt' + uuid.uuid4().hex,
                        'disk_size': random.randint(1, 15000),
                    },
                ],
                'status': {
                    'capacity': {
                        'cpu': random.randint(1, 15),
                        'memory': random.randint(1, 15000),
                        'pods': random.randint(1, 15000),
                    },
                    'allocatable': {
                        'cpu': random.randint(1, 15),
                        'memory': random.randint(1, 15000),
                        'pods': random.randint(1, 15000),
                    },
                    'conditions': []
                },
            },
            'status': 'sts'
        }
        obj = cluster_node.ClusterNode.existing(**object_info)
        return obj



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.sdk.smn.v2 import _proxy
from otcextensions.sdk.smn.v2 import topic
from otcextensions.sdk.smn.v2 import template
from otcextensions.sdk.smn.v2 import subscription
from otcextensions.sdk.smn.v2 import message
from otcextensions.sdk.smn.v2 import sms

from openstack.tests.unit import test_proxy_base


class TestSmnProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super(TestSmnProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestSmnTopic(TestSmnProxy):
    def test_topic_create(self):
        self.verify_create(self.proxy.create_topic, topic.Topic,
                           method_kwargs={'name': 'id'},
                           expected_kwargs={'name': 'id'})

    def test_topic_delete(self):
        self.verify_delete(self.proxy.delete_topic,
                           topic.Topic, True)

    def test_topic_get(self):
        self.verify_get(self.proxy.get_topic, topic.Topic)

    def test_topics(self):
        self.verify_list(self.proxy.topics, topic.Topic)

    def test_topic_update(self):
        self.verify_update(self.proxy.update_topic, topic.Topic)


class TestSmnTemplate(TestSmnProxy):
    def test_template_create(self):
        self.verify_create(self.proxy.create_template, template.Template,
                           method_kwargs={'name': 'id'},
                           expected_kwargs={'name': 'id'})

    def test_template_delete(self):
        self.verify_delete(self.proxy.delete_template,
                           template.Template, True)

    def test_template_get(self):
        self.verify_get(self.proxy.get_template, template.Template)

    def test_templates(self):
        self.verify_list(self.proxy.templates, template.Template)

    def test_template_update(self):
        self.verify_update(self.proxy.update_template, template.Template)


class TestSmnSubscription(TestSmnProxy):
    def test_subscription_create(self):
        self.verify_create(
            self.proxy.create_subscription, subscription.Subscription,
            method_args=['topic_id'],
            method_kwargs={'name': 'id'},
            expected_kwargs={'topic_urn': 'topic_id', 'name': 'id'},
            expected_args=[]
        )

    def test_subscription_delete(self):
        self.verify_delete(self.proxy.delete_subscription,
                           subscription.Subscription, True)

    def test_subscriptions(self):
        self.verify_list(self.proxy.subscriptions, subscription.Subscription)

    def test_topic_subscriptions(self):
        self.verify_list(
            self.proxy.subscriptions, subscription.Subscription,
            method_args=['topic_id'],
            expected_kwargs={'topic_urn': 'topic_id'},
            expected_args=[]
        )


class TestSmnMessage(TestSmnProxy):
    def test_publish_message(self):
        self.verify_create(
            self.proxy.publish_message, message.Message,
            method_args=['topic_id'],
            method_kwargs={'subject': 'Test Message'},
            expected_kwargs={
                'topic_urn': 'topic_id',
                'subject': 'Test Message'},
            expected_args=[]
        )


class TestSmnSms(TestSmnProxy):
    def test_send_sms(self):
        self.verify_create(
            self.proxy.send_sms, sms.Sms,
            method_kwargs={
                'endpoint': '+999999',
                'message': 'Test SMS'
            },
            expected_kwargs={
                'endpoint': '+999999',
                'message': 'Test SMS'
            }
        )



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import mock

from openstackclient.tests.unit import utils

from otcextensions.osclient.dcs.v1 import instance
from otcextensions.tests.unit.osclient.dcs.v1 import fakes


class TestListInstance(fakes.TestDCS):

    objects = fakes.FakeInstance.create_multiple(3)

    columns = ('id', 'name', 'engine', 'status', 'error_code')

    data = []

    for s in objects:
        data.append((
            s.id,
            s.name,
            s.engine,
            s.status,
            s.error_code,
        ))

    def setUp(self):
        super(TestListInstance, self).setUp()

        self.cmd = instance.ListInstance(self.app, None)

        self.client.instances = mock.Mock()

    def test_list(self):
        arglist = [
        ]

        verifylist = [
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.instances.side_effect = [
            self.objects
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.instances.assert_called_once_with(
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))

    def test_list_query(self):
        arglist = [
            '--limit', '1',
            '--start', '2',
            '--name', '3',
            '--status', 'CREATING',
            '--include_failure',
            '--exact_match'
        ]

        verifylist = [
            ('limit', 1),
            ('start', 2),
            ('name', '3'),
            ('status', 'CREATING'),
            ('include_failure', True),
            ('exact_match', True)
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.instances.side_effect = [
            self.objects
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.instances.assert_called_once_with(
            exactMatchName=True,
            includeFailure=True,
            limit=1,
            name='3',
            start=2,
            status='CREATING'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))

    def test_list_bad_arg(self):
        arglist = [
            '--status', 'BAD',
        ]

        verifylist = [
            ('status', 'BAD'),
        ]

        self.assertRaises(
            utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)


class TestBasicInstance(fakes.TestDCS):

    _data = fakes.FakeInstance.create_one()

    columns = (
        'available_zones', 'capacity', 'charging_mode', 'created_at',
        'description', 'engine', 'engine_version', 'error_code',
        'id', 'internal_version', 'ip', 'maintain_begin', 'maintain_end',
        'max_memory', 'name', 'order_id', 'port', 'product_id',
        'resource_spec_code', 'security_group_id', 'security_group_name',
        'status', 'subnet_cidr', 'subnet_id', 'subnet_name',
        'used_memory', 'user_id', 'user_name', 'vpc_id', 'vpc_name')

    data = (
        _data.available_zones,
        _data.capacity,
        _data.charging_mode,
        _data.created_at,
        _data.description,
        _data.engine,
        _data.engine_version,
        _data.error_code,
        _data.id,
        _data.internal_version,
        _data.ip,
        _data.maintain_begin,
        _data.maintain_end,
        _data.max_memory,
        _data.name,
        _data.order_id,
        _data.port,
        _data.product_id,
        _data.resource_spec_code,
        _data.security_group_id,
        _data.security_group_name,
        _data.status,
        _data.subnet_cidr,
        _data.subnet_id,
        _data.subnet_name,
        _data.used_memory,
        _data.user_id,
        _data.user_name,
        _data.vpc_id,
        _data.vpc_name
    )

    def setUp(self):
        super(TestBasicInstance, self).setUp()


class TestShowInstance(TestBasicInstance):

    def setUp(self):
        super(TestShowInstance, self).setUp()

        self.cmd = instance.ShowInstance(self.app, None)

        self.client.find_instance = mock.Mock()

    def test_show_default(self):
        arglist = [
            'name_or_id',
        ]
        verifylist = [
            ('instance', 'name_or_id'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_instance.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_instance.assert_called_with(
            name_or_id='name_or_id'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDeleteInstance(fakes.TestDCS):

    def setUp(self):
        super(TestDeleteInstance, self).setUp()

        self.cmd = instance.DeleteInstance(self.app, None)

        self.client.delete_instance = mock.Mock()

    def test_delete(self):
        arglist = [
            't1',
        ]

        verifylist = [
            ('instance', ['t1'])
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_instance.side_effect = [{}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        calls = [mock.call(instance='t1')]

        self.client.delete_instance.assert_has_calls(calls)
        self.assertEqual(1, self.client.delete_instance.call_count)

    def test_delete_multiple(self):
        arglist = [
            't1',
            't2',
        ]
        verifylist = [
            ('instance', ['t1', 't2'])
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_instance.side_effect = [{}, {}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        calls = [
            mock.call(instance='t1'),
            mock.call(instance='t2')
        ]

        self.client.delete_instance.assert_has_calls(calls)
        self.assertEqual(2, self.client.delete_instance.call_count)


class TestCreateInstance(TestBasicInstance):

    def setUp(self):
        super(TestCreateInstance, self).setUp()

        self.cmd = instance.CreateInstance(self.app, None)

        self.client.create_instance = mock.Mock()

    def test_default(self):
        arglist = [
            '--name', '1',
            '--description', '2',
            '--engine', '3',
            '--engine_version', '4',
            '--capacity', '4',
            '--password', '6',
            '--vpc_id', '7',
            '--security_group_id', '8',
            '--subnet_id', '9',
            '--az', '10',
            '--product_id', 'OTC_DCS_SINGLE',
            '--backup_policy', '{}',
            '--maintain_begin', '11',
            '--maintain_end', '12'
        ]
        verifylist = [
            ('name', '1'),
            ('description', '2'),
            ('engine', '3'),
            ('engine_version', '4'),
            ('password', '6'),
            ('capacity', 4),
            ('vpc_id', '7'),
            ('security_group_id', '8'),
            ('subnet_id', '9'),
            ('az', '10'),
            ('product_id', 'OTC_DCS_SINGLE'),
            ('backup_policy', '{}'),
            ('maintain_begin', '11'),
            ('maintain_end', '12')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.create_instance.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_instance.assert_called_with(
            name='1',
            description='2',
            engine='3',
            engine_version='4',
            capacity=4,
            password='6',
            vpc_id='7',
            security_group_id='8',
            subnet_id='9',
            az='10',
            product_id='OTC_DCS_SINGLE',
            backup_policy='{}',
            maintain_begin='11',
            maintain_end='12'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestSetInstance(TestBasicInstance):

    def setUp(self):
        super(TestSetInstance, self).setUp()

        self.cmd = instance.SetInstance(self.app, None)

        self.client.update_instance = mock.Mock()

    def test_default(self):
        arglist = [
            'inst',
            '--name', '1',
            '--description', '2',
            '--security_group_id', '8',
            '--backup_policy', '{}',
            '--maintain_begin', '11',
            '--maintain_end', '12'
        ]
        verifylist = [
            ('instance', 'inst'),
            ('name', '1'),
            ('description', '2'),
            ('security_group_id', '8'),
            ('backup_policy', '{}'),
            ('maintain_begin', '11'),
            ('maintain_end', '12')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.update_instance.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.update_instance.assert_called_with(
            backup_policy='{}',
            description='2',
            instance='inst',
            maintain_begin='11',
            maintain_end='12',
            name='1',
            security_group_id='8'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestExtendInstance(TestBasicInstance):

    def setUp(self):
        super(TestExtendInstance, self).setUp()

        self.cmd = instance.ExtendInstance(self.app, None)

    def test_default(self):
        arglist = [
            'inst',
            '--capacity', '2',
        ]
        verifylist = [
            ('instance', 'inst'),
            ('capacity', 2),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.extend_instance.side_effect = [
            self._data
        ]
        self.client.find_instance.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_instance.assert_called_with(
            name_or_id='inst',
            ignore_missing=False
        )
        self.client.extend_instance.assert_called_with(
            instance=self._data.id,
            capacity=2,
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestStopInstance(TestBasicInstance):

    def setUp(self):
        super(TestStopInstance, self).setUp()

        self.cmd = instance.StopInstance(self.app, None)

    def test_default(self):
        arglist = [
            'inst',
        ]
        verifylist = [
            ('instance', 'inst'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.stop_instance.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.stop_instance.assert_called_with(
            instance='inst',
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestStartInstance(TestBasicInstance):

    def setUp(self):
        super(TestStartInstance, self).setUp()

        self.cmd = instance.StartInstance(self.app, None)

    def test_default(self):
        arglist = [
            'inst',
        ]
        verifylist = [
            ('instance', 'inst'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.start_instance.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.start_instance.assert_called_with(
            instance='inst',
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestRestartInstance(TestBasicInstance):

    def setUp(self):
        super(TestRestartInstance, self).setUp()

        self.cmd = instance.RestartInstance(self.app, None)

    def test_default(self):
        arglist = [
            'inst',
        ]
        verifylist = [
            ('instance', 'inst'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.restart_instance.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.restart_instance.assert_called_with(
            instance='inst',
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestChangePwdInstance(TestBasicInstance):

    def setUp(self):
        super(TestChangePwdInstance, self).setUp()

        self.cmd = instance.ChangePasswordInstance(self.app, None)

    def test_default(self):
        arglist = [
            'inst',
            '--current_password', 'curr',
            '--new_password', 'new'
        ]
        verifylist = [
            ('instance', 'inst'),
            ('current_password', 'curr'),
            ('new_password', 'new')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.change_instance_password.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.change_instance_password.assert_called_with(
            instance='inst',
            current_password='curr',
            new_password='new'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import resource


class Flavor(resource.Resource):
    base_path = '/node-types'

    resources_key = 'node_types'

    # capabilities
    allow_list = True

    # Properties
    #: Availability Zones
    availability_zones = resource.Body('availableZones')
    #: Node type details
    detail = resource.Body('detail', type=list, list_type=dict)
    #: Disk Size *Type: int*
    disk_size = resource.Body('disk_size', type=int)
    #: Disk Type *Type: str*
    disk_type = resource.Body('disk_type')
    #: Name of a node type
    name = resource.Body('spec_name')
    #: ram. *Type: int*
    ram = resource.Body('mem', type=int)
    #: Name of a node type
    spec_name = resource.Body('spec_name')
    #: cpu count. *Type: int*
    vcpu = resource.Body('vCPU', type=int)

    @classmethod
    def list(cls, session, paginated=True, base_path=None, **params):
        """This method is a generator which yields resource objects.

        This resource object list generator handles pagination and takes query
        params for response filtering.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param bool paginated: ``True`` if a GET to this resource returns
                               a paginated series of responses, or ``False``
                               if a GET returns only one page of data.
                               **When paginated is False only one
                               page of data will be returned regardless
                               of the API's support of pagination.**
        :param str base_path: Base part of the URI for listing resources, if
                              different from
                              :data:`~openstack.resource.Resource.base_path`.
        :param dict params: These keyword arguments are passed through the
            :meth:`~openstack.resource.QueryParamter._transpose` method
            to find if any of them match expected query parameters to be
            sent in the *params* argument to
            :meth:`~keystoneauth1.adapter.Adapter.get`. They are additionally
            checked against the
            :data:`~openstack.resource.Resource.base_path` format string
            to see if any path fragments need to be filled in by the contents
            of this argument.

        :return: A generator of :class:`Resource` objects.
        :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
                 :data:`Resource.allow_list` is not set to ``True``.
        :raises: :exc:`~openstack.exceptions.InvalidResourceQuery` if query
                 contains invalid params.
        """
        if not cls.allow_list:
            raise exceptions.MethodNotSupported(cls, 'list')
        session = cls._get_session(session)
        microversion = cls._get_microversion(session)

        if base_path is None:
            base_path = cls.base_path
        cls._query_mapping._validate(params, base_path=base_path)
        query_params = cls._query_mapping._transpose(params, cls)
        uri = base_path % params

        # Copy query_params due to weird mock unittest interactions
        response = session.get(
            uri,
            headers={'Accept': 'application/json'},
            params=query_params.copy(),
            microversion=microversion,
        )
        exceptions.raise_from_response(response)

        data = response.json()
        resources = data[cls.resources_key]
        if not isinstance(resources, list):
            resources = [resources]

        for raw_resource in resources:
            raw_resource.pop('self', None)

            for detail in raw_resource['detail']:
                if detail['type'].lower() in (
                    'ssd',
                    'sata',
                    'essd',
                    'co-p1',
                    'uh-l1',
                ):
                    raw_resource['disk_type'] = detail['type']
                    raw_resource['disk_size'] = detail['value']
                else:
                    raw_resource[detail['type']] = detail['value']
            value = cls.existing(
                microversion=microversion,
                connection=session._get_connection(),
                **raw_resource,
            )
            yield value
        else:
            return



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
# import datetime
import random

import mock
import time
import uuid

from otcextensions.tests.unit.osclient import test_base

from otcextensions.sdk.anti_ddos.v1 import config
from otcextensions.sdk.anti_ddos.v1 import floating_ip
from otcextensions.sdk.anti_ddos.v1 import status


class TestAntiDDoS(test_base.TestCommand):

    def setUp(self):
        super(TestAntiDDoS, self).setUp()

        self.app.client_manager.anti_ddos = mock.Mock()
        self.client = self.app.client_manager.anti_ddos


class FakeConfig(test_base.Fake):
    """Fake one or more Config"""

    @classmethod
    def generate(cls):
        object_info = {
            'traffic_limited_list': [
                {
                    'traffic_pos_id': i,
                    'traffic_per_second': random.randint(1, 600),
                    'packet_per_second': random.randint(1, 600),
                } for i in range(1, random.randint(1, 5))
            ],
            'http_limited_list': [
                {
                    'http_request_pos_id': i,
                    'http_packet_per_second': random.randint(1, 600),
                } for i in range(1, random.randint(1, 5))
            ],
            'connection_limited_list': [
                {
                    'cleaning_access_pos_id': i,
                    'new_connection_limited': random.randint(1, 600),
                    'total_connection_limited': random.randint(1, 600),
                } for i in range(1, random.randint(1, 5))
            ],
        }
        obj = config.Config.existing(**object_info)
        return obj


class FakeFloatingIP(test_base.Fake):
    """Fake one or more Floating IP"""

    @classmethod
    def generate(cls):
        object_info = {
            'floating_ip_address': uuid.uuid4().hex,
            'floating_ip_id': uuid.uuid4().hex,
            'network_type': random.choice(['EIP', 'ELB']),
            'status': random.choice(['normal', 'configging', 'notConfig']),
            'is_enable_l7': random.choice([False, True]),
            'app_type_id': random.randint(1, 5),
            'cleaning_access_pos_id': random.randint(1, 5),
            'http_request_pos_id': random.randint(1, 5),
        }
        obj = floating_ip.FloatingIP.existing(**object_info)
        return obj


class FakeFloatingIPEvent(test_base.Fake):
    """Fake one or more Floating IP events"""

    @classmethod
    def generate(cls):
        object_info = {
            'floating_ip_id': uuid.uuid4().hex,
            'start_time': time.time() * 1000,
            'end_time': time.time() * 1000,
            'trigger_bps': random.randint(1, 5),
            'trigger_pps': random.randint(1, 5),
            'trigger_http_pps': random.randint(1, 5),
            'status': random.choice([1, 2]),
        }
        obj = status.FloatingIPEvent.existing(**object_info)
        return obj


class FakeFloatingIPStatDay(test_base.Fake):
    """Fake one or more Floating IP stat day"""

    @classmethod
    def generate(cls):
        object_info = {
            'floating_ip_id': uuid.uuid4().hex,
            'period_start': time.time() * 1000,
            'bps_in': random.randint(1, 5),
            'bps_attack': random.randint(1, 5),
            'total_bps': random.randint(1, 5),
            'pps_in': random.randint(1, 5),
            'pps_attack': random.randint(1, 5),
            'total_pps': random.randint(1, 5),
        }
        obj = status.FloatingIPDayStat.existing(**object_info)
        return obj


class FakeFloatingIPStatWeek(test_base.Fake):
    """Fake one or more Floating IP stat week"""

    @classmethod
    def generate(cls):
        object_info = {
            'weekdata': [{
                'period_start': time.time() * 1000,
                'ddos_intercept_times': random.randint(1, 5),
                'ddos_blackhole_times': random.randint(1, 5),
                'max_attack_bps': random.randint(1, 5),
                'max_attack_conns': random.randint(1, 5),
            }]
        }
        obj = status.FloatingIPWeekStat.existing(**object_info)
        return obj



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid
from otcextensions.sdk.function_graph.v2 import function
from otcextensions.tests.functional import base

from openstack import _log

_logger = _log.setup_logging('openstack')


class TestFunction(base.BaseFunctionalTest):
    ID = None
    uuid = uuid.uuid4().hex[:8]

    def setUp(self):
        super(TestFunction, self).setUp()
        self.attrs = {
            'func_name': 'test-function-' + self.uuid,
            'package': 'default',
            'runtime': 'Python3.9',
            'handler': 'index.handler',
            'timeout': 30,
            'memory_size': 128,
            'code_type': 'inline',
        }

        self.NAME = 'test-function-' + self.uuid
        self.UPDATE_NAME = 'test-function-upd-' + self.uuid

        self.function = self.conn.functiongraph.create_function(**self.attrs)
        assert isinstance(self.function, function.Function)
        self.assertEqual(self.NAME, self.function.func_name)
        self.ID = self.function.func_id
        self.addCleanup(self.conn.functiongraph.delete_function, self.function)

    def test_get_function_code(self):
        found = self.conn.functiongraph.get_function_code(self.function)
        self.assertEqual(found.code_type, self.attrs['code_type'])
        self.assertEqual(found.runtime, self.attrs['runtime'])

    def test_get_function_metadata(self):
        found = self.conn.functiongraph.get_function_metadata(self.function)
        self.assertEqual(found.is_stateful_function, False)
        self.assertEqual(found.enable_dynamic_memory, False)

    def test_resource_tags(self):
        self.conn.functiongraph.create_resource_tags(
            self.function,
            tags=[
                {
                    'key': 'key',
                    'value': 'value'
                },
                {
                    'key': 'testKey2',
                    'value': 'testValue2'
                }
            ]
        )
        found = self.conn.functiongraph.get_resource_tags(self.function)
        self.assertEqual(len(found.tags), 2)
        self.conn.functiongraph.delete_resource_tags(
            self.function,
            tags=[
                {
                    'key': 'testKey2',
                    'value': 'testValue2'
                }
            ]
        )
        found = self.conn.functiongraph.get_resource_tags(self.function)
        self.assertEqual(len(found.tags), 1)

    def test_list_functions(self):
        functions = list(self.conn.functiongraph.functions())
        self.assertIn(self.NAME, functions[0].func_name)

    def test_update_function(self):
        code_attrs = {
            'code_filename': 'index.zip',
            'code_type': 'inline',
            'func_code': {
                'file': 'UEsDBAoAAAAIAPQ1M1gNImPLrAAAAAEBAAAIAAAAaW5kZXgucHlN'
                        'jtEOgjAMRd/5igVfxDAlxhjDo0S/wB+YrMgMdMvWGYnh390wEfrU'
                        '3nvb0xXjG85qLRU+Sk8NP0UhUb3RltjTaUwkNKwVKDuwbA0vQMrD'
                        'AhK8KSsTFsoCeYvsMw2xUkeCvKu0hLRk+6LIZ0u5s3BwPFwwUEEG'
                        '/yo6B4vEXcshyBG+lb437kfNFpEWhATrQmqGTkYVH0Pit8FEdCqM'
                        '6VQtSGncxYPpPz5O3fgFUEsBAh4DCgAAAAgA9DUzWA0iY8usAAAA'
                        'AQEAAAgAAAAAAAAAAAAAAPMCAAAAAGluZGV4LnB5UEsFBgAAAAAB'
                        'AAEANgAAANIAAAAAAA=='
            }
        }
        code = self.conn.functiongraph.update_function_code(
            self.function, **code_attrs)
        self.assertIsNotNone(code)
        self.assertEqual(code.code_type, code_attrs['code_type'])

        metadata_attrs = {
            'memory_size': 768,
            'handler': 'index.handler',
            'runtime': 'Python3.9',
            'mount_config': {
                'mount_user': {
                    'user_id': -1,
                    'user_group_id': -1
                },
            },
            'timeout': 40
        }
        metadata = self.conn.functiongraph.update_function_metadata(
            self.function, **metadata_attrs)
        self.assertIsNotNone(metadata)
        self.assertEqual(metadata.handler, metadata_attrs['handler'])
        self.assertEqual(metadata.memory_size, metadata_attrs['memory_size'])
        self.assertEqual(metadata.runtime, metadata_attrs['runtime'])
        self.assertEqual(metadata.timeout, metadata_attrs['timeout'])

        instances = self.conn.functiongraph.update_max_instances(
            self.function, 300)
        self.assertIsNotNone(instances)
        self.assertEqual(metadata.strategy_config['concurrency'], 300)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''Instance v1 action implementations'''
import argparse
import logging

from osc_lib import exceptions
from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import sdk_utils
from otcextensions.i18n import _


LOG = logging.getLogger(__name__)


def _get_columns(item):
    column_map = {
    }
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


def set_attributes_for_print(instances):
    for instance in instances:
        setattr(instance, 'flavor_id', instance.flavor['id'])
        if getattr(instance, 'volume', None):
            setattr(instance, 'size', instance.volume['size'])
        else:
            setattr(instance, 'size', '-')
        datastore = instance.datastore
        if datastore.get('version'):
            setattr(instance, 'datastore_version',
                    datastore['version'])
        if datastore.get('type'):
            setattr(instance, 'datastore_type', datastore['type'])
        setattr(instance, 'type', instance.type)
        yield instance


def set_attributes_for_print_detail(instance):
    info = {}  # instance._info.copy()
    info['name'] = instance.name
    info['flavor_id'] = instance.flavor['id']
    if getattr(instance, 'volume', None):
        info['volume'] = instance.volume['size']
        if 'used' in instance.volume:
            info['volume_used'] = instance.volume['used']
    if getattr(instance, 'ip', None):
        info['ip'] = ', '.join(instance.ip)
    if getattr(instance, 'datastore', None):
        info['datastore'] = instance.datastore['type']
        info['datastore_version'] = instance.datastore['version']
    if getattr(instance, 'configuration', None):
        info['configuration'] = instance.configuration['id']
    if getattr(instance, 'replica_of', None):
        info['replica_of'] = instance.replica_of['id']
    if getattr(instance, 'replicas', None):
        replicas = [replica['id'] for replica in instance.replicas]
        info['replicas'] = ', '.join(replicas)
    if getattr(instance, 'networks', None):
        info['networks'] = instance.networks['name']
        info['networks_id'] = instance.networks['id']
    if getattr(instance, 'fault', None):
        info.pop('fault', None)
        info['fault'] = instance.fault['message']
        info['fault_date'] = instance.fault['created']
        if 'details' in instance.fault and instance.fault['details']:
            info['fault_details'] = instance.fault['details']
    info.pop('links', None)
    return info


class ListDatabaseInstances(command.Lister):
    _description = _('List database instances')
    columns = ['ID', 'Name', 'Datastore Type', 'Datastore Version', 'Status',
               'Flavor ID', 'Type', 'Size', 'Region']

    def get_parser(self, prog_name):
        parser = super(ListDatabaseInstances, self).get_parser(prog_name)
        parser.add_argument(
            '--limit',
            dest='limit',
            metavar='<limit>',
            type=int,
            default=None,
            help=_('Limit the number of results displayed. (Not supported)')
        )
        parser.add_argument(
            '--marker',
            dest='marker',
            metavar='<ID>',
            help=_('Begin displaying the results for IDs greater than the '
                   'specified marker. When used with --limit, set this to '
                   'the last ID displayed in the previous run. '
                   '(Not supported)')
        )
        parser.add_argument(
            '--include_clustered', '--include-clustered',
            dest='include_clustered',
            action='store_true',
            default=False,
            help=_('Include instances that are part of a cluster '
                   '(default %(default)s).  --include-clustered may be '
                   'deprecated in the future, retaining just '
                   '--include_clustered.'
                   '(Not supported)')
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds

        data = client.instances()

        if data:
            data = set_attributes_for_print(data)

        return (
            self.columns,
            (utils.get_item_properties(
                s,
                self.columns,
            ) for s in data)
        )


class ShowDatabaseInstance(command.ShowOne):
    _description = _("Show instance details")

    columns = ['ID', 'Name', 'Datastore', 'Datastore Version', 'Status',
               'Flavor ID', 'Size', 'Region']

    def get_parser(self, prog_name):
        parser = super(ShowDatabaseInstance, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('Instance (name or ID)'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds
        obj = client.find_instance(parsed_args.instance)

        # data = set_attributes_for_print_detail(obj)
        #
        # return (
        #     self.columns,
        #     (utils.get_dict_properties(
        #         s,
        #         self.columns,
        #     ) for s in data)
        # )

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters={})
        #
        return (display_columns, data)

        #
        # instance = set_attributes_for_print_detail(obj)
        # return zip(*sorted(instance.items()))


class CreateDatabaseInstance(command.ShowOne):

    _description = _("Creates a new database instance.")

    def get_parser(self, prog_name):
        parser = super(CreateDatabaseInstance, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_("Name of the instance."),
        )
        parser.add_argument(
            'flavor',
            metavar='<flavor>',
            help=_("A flavor ID or name."),
        )
        parser.add_argument(
            '--size',
            metavar='<size>',
            type=int,
            required=True,
            help=_("Size of the instance disk volume in GB. "
                   "Required when volume support is enabled."),
        )
        parser.add_argument(
            '--volume_type',
            metavar='<volume_type>',
            type=str,
            default=None,
            choices=['COMMON', 'ULTRAHIGH'],
            help=_("Volume type. (COMMON, ULTRAHIGH)."),
        )
        parser.add_argument(
            '--databases',
            metavar='<database>',
            nargs="+",
            default=[],
            help=_("Optional list of databases. Not Supported"),
        )
        parser.add_argument(
            '--users',
            metavar='<user:password>',
            nargs="+",
            required=True,
            help=_("list of users."),
        )
        parser.add_argument(
            '--backup',
            metavar='<backup>',
            default=None,
            help=_("A backup name or ID."),
        )
        parser.add_argument(
            '--availability_zone',
            metavar='<availability_zone>',
            default=None,
            help=_("The Zone hint to give to Nova."),
        )
        parser.add_argument(
            '--datastore',
            metavar='<datastore>',
            default=None,
            help=_("A datastore name or ID."),
        )
        parser.add_argument(
            '--datastore_version',
            metavar='<datastore_version>',
            default=None,
            help=_("A datastore version name or ID."),
        )
        # API doc claims subnet, but it is network
        parser.add_argument(
            '--nic',
            metavar='<net-id=<net-uuid>,v4-fixed-ip=<ip-addr>,'
                    'port-id=<port-uuid>>',
            action='append',
            dest='nics',
            default=[],
            help=_("Create a NIC on the instance. Specify option multiple "
                   "times to create multiple NICs. net-id: attach NIC to "
                   "network with this ID (either port-id or net-id must be "
                   "specified), v4-fixed-ip: IPv4 fixed address for NIC "
                   "(optional), port-id: attach NIC to port with this ID "
                   "(either port-id or net-id must be specified)."),
        )
        parser.add_argument(
            '--configuration',
            metavar='<configuration>',
            default=None,
            help=_("ID of the configuration group to attach to the instance."),
        )
        parser.add_argument(
            '--replica_of',
            metavar='<source_instance>',
            default=None,
            help=_("ID or name of an existing instance to replicate from."),
        )
        parser.add_argument(
            '--replica_count',
            metavar='<count>',
            type=int,
            default=None,
            help=_("Number of replicas to create (defaults to 1 if "
                   "replica_of specified)."
                   "(None or 1 is allowed)"),
        )
        parser.add_argument(
            '--module',
            metavar='<module>',
            type=str,
            dest='modules',
            action='append',
            default=[],
            help=_("ID or name of the module to apply.  Specify multiple "
                   "times to apply multiple modules."
                   "(Not supported)"),
        )
        parser.add_argument(
            '--locality',
            metavar='<policy>',
            default=None,
            choices=['affinity', 'anti-affinity'],
            help=_("Locality policy to use when creating replicas. Choose "
                   "one of %(choices)s."
                   "(Not supported)"),
        )
        parser.add_argument(
            '--region',
            metavar='<region>',
            type=str,
            default=None,
            help=argparse.SUPPRESS,
        )
        parser.add_argument(
            '--network_id',
            metavar='<network_id>',
            type=str,
            # required=True,
            help=_('Network (VPC) ID')
        )
        # parser.add_argument(
        #     '--subnet_id',
        #     metavar='<subnet_id>',
        #     type=str,
        #     # required=True,
        #     help=_('Subnet ID')
        # )
        parser.add_argument(
            '--security_group',
            metavar='<security_group>',
            type=str,
            # required=True,
            help=_('Security group ID to be associated with NIC')
        )
        return parser

    def take_action(self, parsed_args):
        # raise NotImplementedError
        # Attention: not conform password result in BadRequest with no info
        client = self.app.client_manager.rds

        attrs = {}
        attrs['flavorRef'] = client.find_flavor(parsed_args.flavor).id
        volume = None
        if parsed_args.size is not None and parsed_args.size <= 0:
            raise exceptions.ValidationError(
                _("Volume size '%s' must be an integer and greater than 0.")
                % parsed_args.size)
        elif parsed_args.size:
            volume = {"size": parsed_args.size}
            if parsed_args.volume_type:
                volume['type'] = parsed_args.volume_type
            attrs['volume'] = volume
        # restore_point = None
        if parsed_args.backup:
            pass
            # TODO(agoncharov)
        #     restore_point = {"backupRef": osc_utils.find_resource(
        #         database.backups, parsed_args.backup).id}
        replica_count = parsed_args.replica_count
        if parsed_args.replica_of:
            pass
            # TODO(agoncharov)
            attrs['replica_of'] = client.find_instance(parsed_args.replica_of)
            attrs['replica_count'] = replica_count or 1
        # locality = None
        # if parsed_args.locality:
        #     locality = parsed_args.locality
        #     if replica_of:
        #         raise exceptions.ValidationError(
        #             _('Cannot specify locality when adding replicas '
        #               'to existing master.'))
        attrs['users'] = [{'name': n, 'password': p} for (n, p) in
                          [z.split(':')[:2] for z in parsed_args.users]]
        nics = []
        for nic_str in parsed_args.nics:
            nic_info = dict([(k, v) for (k, v) in [z.split("=", 1)[:2] for z in
                                                   nic_str.split(",")]])
            # need one or the other, not both, not none (!= ~ XOR)
            if not (bool(nic_info.get('net-id')) != bool(
                    nic_info.get('port-id'))):
                raise exceptions.\
                    ValidationError(_("Invalid NIC argument: %s. Must specify "
                                      "either net-id or port-id but not both. "
                                      "Please refer to help.")
                                    % (_("nic='%s'") % nic_str))
            if parsed_args.security_group:
                nic_info['securityGroupId'] = parsed_args.security_group
            nics.append(nic_info)
        if len(nics):
            attrs['nics'] = nics
        # modules = []
        # for module in parsed_args.modules:
        #     modules.append(osc_utils.find_resource(database.modules,
        #                                            module).id)

        datastore = {
            'type': parsed_args.datastore,
            'version': parsed_args.datastore_version
        }
        attrs['datastore'] = datastore
        attrs['name'] = parsed_args.name
        if parsed_args.availability_zone:
            attrs['availability_zone'] = parsed_args.availability_zone
        if parsed_args.configuration:
            attrs['configuration'] = parsed_args.configuration
        if parsed_args.region:
            attrs['region_name'] = parsed_args.region
        if parsed_args.network_id:
            attrs['vpc'] = parsed_args.network_id
        instance = client.create_instance(**attrs)
        instance = set_attributes_for_print_detail(instance)
        return zip(*sorted(instance.items()))


class CreateInstanceFromBackup(command.ShowOne):
    _description = _("Creates a new database instance from backup.")

    def get_parser(self, prog_name):
        parser = super(CreateInstanceFromBackup, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_("Name of the instance."),
        )
        parser.add_argument(
            '--flavor',
            metavar='<flavor>',
            required=True,
            help=_("A flavor ID or name."),
        )
        parser.add_argument(
            '--size',
            metavar='<size>',
            type=int,
            required=True,
            help=_("Size of the instance disk volume in GB. "
                   "Required when volume support is enabled."),
        )
        parser.add_argument(
            '--enable_ha',
            action='store_true',
            help=_('Specifies the HA configuration parameter.\n'
                   'The value `true` indicates creating a primary/standby '
                   'DB instance. The value `false` indicates creating '
                   'a single DB instance.'),
        )
        parser.add_argument(
            '--replication_mode',
            metavar='<replication_mode>',
            default=None,
            choices=('async', 'sync', 'semisync'),
            help=_('Specifies the replication mode for the standby DB '
                   'instance. Valid values:'
                   '* `async`: indicates the asynchronous replication mode.'
                   '* `sync`: indicates the synchronous replication mode.'
                   '* `semisync`: indicates the semisynchronous '
                   'replication mode.'),
        )
        parser.add_argument(
            '--backup',
            metavar='<backup>',
            help=_("ID or name of the backup to create instance from."),
        )
        parser.add_argument(
            '--si',
            metavar='<si>',
            help=_("ID or name of the backup to create instance from."),
        )

        return parser

    def take_action(self, parsed_args):
        # raise NotImplementedError
        # Attention: not conform password result in BadRequest with no info
        client = self.app.client_manager.rds

        attrs = {}
        # attrs['flavorRef'] = client.find_flavor(parsed_args.flavor).id
        attrs['flavorRef'] = parsed_args.flavor
        volume = None
        if parsed_args.size is not None and parsed_args.size <= 0:
            raise exceptions.ValidationError(
                _("Volume size '%s' must be an integer and greater than 0.")
                % parsed_args.size)
        elif parsed_args.size:
            volume = {"size": parsed_args.size}
            attrs['volume'] = volume
        if parsed_args.enable_ha or parsed_args.replication_mode:
            attrs['ha'] = {
                'enable': True,
                'replicationMode': parsed_args.replication_mode
            }
        # restore_point = None
        if parsed_args.backup:
            attrs['restore_point'] = {'backupRef': parsed_args.backup}
            if parsed_args.si:
                attrs['restore_point']['sourceInstanceId'] = parsed_args.si
        attrs['name'] = parsed_args.name
        instance = client.create_instance(**attrs)
        instance = set_attributes_for_print_detail(instance)
        return zip(*sorted(instance.items()))


class DeleteDatabaseInstance(command.Command):

    _description = _("Deletes an instance.")

    def get_parser(self, prog_name):
        parser = super(DeleteDatabaseInstance, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('ID or name of the Instance'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds
        try:
            client.delete_instance(parsed_args.instance)
        except Exception as e:
            msg = (_("Failed to delete instance %(instance)s: %(e)s")
                   % {'instance': parsed_args.instance, 'e': e})
            raise exceptions.CommandError(msg)


class ResetDatabaseInstanceStatus(command.Command):

    _description = _("Set the task status of an instance to NONE if the "
                     "instance is in BUILD or ERROR state. Resetting task "
                     "status of an instance in BUILD state will allow "
                     "the instance to be deleted.")

    def get_parser(self, prog_name):
        parser = super(ResetDatabaseInstanceStatus, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('ID or name of the instance'),
        )
        return parser

    def take_action(self, parsed_args):
        raise NotImplementedError
        # db_instances = self.app.client_manager.database.instances
        # instance = osc_utils.find_resource(db_instances,
        #                                    parsed_args.instance)
        # db_instances.reset_status(instance)


class ResizeDatabaseInstanceFlavor(command.Command):

    _description = _("Resize an instance with a new flavor")

    def get_parser(self, prog_name):
        parser = super(ResizeDatabaseInstanceFlavor, self).get_parser(
            prog_name
        )
        parser.add_argument(
            'instance',
            metavar='<instance>',
            type=str,
            help=_('ID or name of the instance')
        )
        parser.add_argument(
            'flavor_id',
            metavar='<flavor_id>',
            type=str,
            help=_('New flavor of the instance')
        )
        return parser

    def take_action(self, parsed_args):
        # raise NotImplementedError
        client = self.app.client_manager.rds
        instance = client.find_instance(parsed_args.instance)
        flavor = client.find_flavor(
            parsed_args.flavor_id,
            ignore_missing=False)

        instance.resize(client, flavor.id)


class UpgradeDatabaseInstance(command.Command):

    _description = _("Upgrades an instance to a new datastore version.")

    def get_parser(self, prog_name):
        parser = super(UpgradeDatabaseInstance, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            type=str,
            help=_('ID or name of the instance.'),
        )
        parser.add_argument(
            'datastore_version',
            metavar='<datastore_version>',
            help=_('ID or name of the instance.'),
        )
        return parser

    def take_action(self, parsed_args):
        raise NotImplementedError
        # db_instances = self.app.client_manager.database.instances
        # instance = osc_utils.find_resource(db_instances,
        #                                    parsed_args.instance)
        # db_instances.upgrade(instance, parsed_args.datastore_version)


class EnableDatabaseInstanceLog(command.ShowOne):

    _description = _("Instructs Trove guest to start collecting log details.")

    def get_parser(self, prog_name):
        parser = super(EnableDatabaseInstanceLog, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            type=str,
            help=_('Id or Name of the instance.')
        )
        parser.add_argument(
            'log_name',
            metavar='<log_name>',
            type=str,
            help=_('Name of log to publish.')
        )
        return parser

    def take_action(self, parsed_args):
        raise NotImplementedError
        # db_instances = self.app.client_manager.database.instances
        # instance = osc_utils.find_resource(db_instances,
        #                                    parsed_args.instance)
        # log_info = db_instances.log_enable(instance, parsed_args.log_name)
        # result = log_info._info
        # return zip(*sorted(result.items()))


class ResizeDatabaseInstanceVolume(command.Command):

    _description = _("Resizes the volume size of an instance.")

    def get_parser(self, prog_name):
        parser = super(ResizeDatabaseInstanceVolume, self).get_parser(
            prog_name
        )
        parser.add_argument(
            'instance',
            metavar='<instance>',
            type=str,
            help=_('ID or name of the instance.')
        )
        parser.add_argument(
            'size',
            metavar='<size>',
            type=int,
            default=None,
            help=_('New size of the instance disk volume in GB.')
        )
        return parser

    def take_action(self, parsed_args):
        raise NotImplementedError
        # db_instances = self.app.client_manager.database.instances
        # instance = osc_utils.find_resource(db_instances,
        #                                    parsed_args.instance)
        # db_instances.resize_volume(instance, parsed_args.size)


class ForceDeleteDatabaseInstance(command.Command):

    _description = _("Force delete an instance.")

    def get_parser(self, prog_name):
        parser = (super(ForceDeleteDatabaseInstance, self)
                  .get_parser(prog_name))
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('ID or name of the instance'),
        )
        return parser

    def take_action(self, parsed_args):
        raise NotImplementedError
        # db_instances = self.app.client_manager.database.instances
        # instance = osc_utils.find_resource(db_instances,
        #                                    parsed_args.instance)
        # db_instances.reset_status(instance)
        # try:
        #     db_instances.delete(instance)
        # except Exception as e:
        #     msg = (_("Failed to delete instance %(instance)s: %(e)s")
        #            % {'instance': parsed_args.instance, 'e': e})
        #     raise exceptions.CommandError(msg)


class RestartDatabaseInstance(command.Command):

    _description = _("Restarts an instance.")

    def get_parser(self, prog_name):
        parser = super(RestartDatabaseInstance, self).get_parser(
            prog_name
        )
        parser.add_argument(
            'instance',
            metavar='<instance>',
            type=str,
            help=_('ID or name of the instance.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds

        client.restart_instance(parsed_args.instance)


class RestoreDatabaseInstance(command.Command):

    _description = _("Restores an instance from backup.")

    def get_parser(self, prog_name):
        parser = super(RestoreDatabaseInstance, self).get_parser(
            prog_name
        )
        parser.add_argument(
            'instance',
            metavar='<instance>',
            type=str,
            help=_('ID or name of the instance.')
        )
        parser.add_argument(
            '--backup',
            metavar='<backup>',
            type=str,
            help=_('ID or name of the backup.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds

        client.restore_instance(
            instance=parsed_args.instance,
            backup=parsed_args.backup)


class UpdateDatabaseInstance(command.Command):

    _description = _("Updates an instance: Edits name, "
                     "configuration, or replica source.")

    def get_parser(self, prog_name):
        parser = super(UpdateDatabaseInstance, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            type=str,
            help=_('ID or name of the instance.'),
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            type=str,
            default=None,
            help=_('ID or name of the instance.'),
        )
        parser.add_argument(
            '--configuration',
            metavar='<configuration>',
            type=str,
            default=None,
            help=_('ID of the configuration reference to attach.'),
        )
        parser.add_argument(
            '--detach_replica_source',
            '--detach-replica-source',
            dest='detach_replica_source',
            action="store_true",
            default=False,
            help=_('Detach the replica instance from its replication source. '
                   '--detach-replica-source may be deprecated in the future '
                   'in favor of just --detach_replica_source'),
        )
        parser.add_argument(
            '--remove_configuration',
            dest='remove_configuration',
            action="store_true",
            default=False,
            help=_('Drops the current configuration reference.'),
        )
        return parser

    def take_action(self, parsed_args):
        raise NotImplementedError
        # db_instances = self.app.client_manager.database.instances
        # instance = osc_utils.find_resource(db_instances,
        #                                    parsed_args.instance)
        # db_instances.edit(instance, parsed_args.configuration,
        #                   parsed_args.name,
        #                   parsed_args.detach_replica_source,
        #                   parsed_args.remove_configuration)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base
from otcextensions.sdk.apig.v2 import app


EXAMPLE_QUOTA = {
    'gateway_id': 'gateway-67890',
    'app_id': 'app-12345',
    'app_quota_id': 'quota-123',
    'name': 'Test Quota',
    'call_limits': 1000,
    'time_unit': 'minute',
    'time_interval': 60,
    'remark': 'This is a test quota',
    'reset_time': '2025-02-07T12:00:00Z',
    'create_time': '2025-02-07T12:30:00Z',
    'bound_app_num': 5
}


class TestQuota(base.TestCase):

    def test_basic(self):
        sot = app.Quota()
        self.assertEqual('/apigw/instances/%(gateway_id)s/apps/'
                         '%(app_id)s/bound-quota',
                         sot.base_path)
        self.assertTrue(sot.allow_fetch)

    def test_make_it(self):
        sot = app.Quota(**EXAMPLE_QUOTA)
        self.assertEqual(EXAMPLE_QUOTA['gateway_id'], sot.gateway_id)
        self.assertEqual(EXAMPLE_QUOTA['app_id'], sot.app_id)
        self.assertEqual(EXAMPLE_QUOTA['app_quota_id'], sot.app_quota_id)
        self.assertEqual(EXAMPLE_QUOTA['name'], sot.name)
        self.assertEqual(EXAMPLE_QUOTA['call_limits'], sot.call_limits)
        self.assertEqual(EXAMPLE_QUOTA['time_unit'], sot.time_unit)
        self.assertEqual(EXAMPLE_QUOTA['time_interval'], sot.time_interval)
        self.assertEqual(EXAMPLE_QUOTA['remark'], sot.remark)
        self.assertEqual(EXAMPLE_QUOTA['reset_time'], sot.reset_time)
        self.assertEqual(EXAMPLE_QUOTA['create_time'], sot.create_time)
        self.assertEqual(EXAMPLE_QUOTA['bound_app_num'], sot.bound_app_num)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import openstack
from otcextensions import sdk
"""
Create an multipart upload of a large object
"""

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')
sdk.register_otc_extensions(conn)
sdk.get_ak_sk(conn)

container = conn.obs.create_container(
    name='test-multipart',
    storage_acl='private',
    storage_class='STANDARD'
)

obj = conn.obs.upload_object(
    name='pdfs/book.pdf',
    filename='my-book.pdf',
    container=container,
    segment_size=1400000
)
